(reset)
(import "de.uni_kassel.usf.LandReclamationJess.SandboxAgent")
(import "de.uni_kassel.usf.LandReclamationJess.LandParcel")
(import "de.uni_kassel.usf.LandReclamationJess.Space")
(import "de.uni_kassel.usf.LandReclamationJess.Model")
(import nrc.fuzzy.*)
(import nrc.fuzzy.jess.*)
(load-package FuzzyFunctions)


(defclass Model Model)
(defclass Space Space)
(defclass agent SandboxAgent)
(defclass LandParcel LandParcel)


(deftemplate agent-with-lrs-rule
    "agent and their LRS maintenance rules"
    (slot agent (type object))
    (slot ruleNo)
    (slot lastChange))

(defrule assign-rule-to-agent
     ""
    (Model (tick ?t&:(= ?t 1)))
    (agent (ID ?id)(LRSRule ?r)(OBJECT ?obj))
        =>
    (assert (agent-with-lrs-rule (agent ?obj)(ruleNo ?r)(lastChange ?t)))
	(printout t ?t ": agent " ?id " uses rule " ?r "." crlf)
)    	

(defrule adjust-rating
    ""
;;    (Model (month 12) (tick ?t))
    (Model (tick ?t))
    ?awr <- (agent-with-lrs-rule (agent ?obj) (ruleNo ?rn) (lastChange ?lc&:(> ?t ?lc)))
    (agent
        (estimationDue TRUE)
        (OBJECT ?obj))
        =>
    	(bind ?id (?obj getID))
    	(printout t ?t ": agent " ?id " adjusts rating." crlf)
        (?obj estimateRating)
    	(?obj setEstimationDue FALSE)
    	(bind ?new-rn (?obj getLRSRule))
    	(if (<> ?new-rn ?rn) then
    		(if (= ?new-rn 0.0) then (?obj setLrsCostSubsidy 0.0) else (?obj payLrsCostSubsidy))    
            (printout t ?t ": agent " ?id " changes from " ?rn " to " ?new-rn crlf)
        	(retract ?awr)
        	(assert (agent-with-lrs-rule (agent ?obj) (ruleNo ?new-rn) (lastChange ?t)))
        )
)

(defrule do-not-maintain-lrs
    "if rule is 0 stop maintaining immediately"
    (Model
        (tick ?t))
    (agent-with-lrs-rule (agent ?obj) (ruleNo 0))
    (agent
        (ID ?id)
        (maintainLRS TRUE)
        (OBJECT ?obj))
        =>
        (?obj setMaintainLRS FALSE)
    	
        (printout t ?t ": agent " ?id " stops maintaining LRS." crlf)
        )

(defrule start-maintain-lrs-1
    "if yieldBalance gets low, decide to maintain lrs"
    (Model
        (tick ?t))
    (agent-with-lrs-rule (agent ?obj) (ruleNo 1))
    (agent
        (ID ?id)
        (maintainLRS FALSE)
        (yieldBalancePerception ?y&:(fuzzy-match ?y "low"))
        (OBJECT ?obj))
        =>
        (?obj setMaintainLRS TRUE)
;;        (printout t ?t ": agent " ?id " decides to maintain LRS." crlf)
)

(defrule stop-maintain-lrs-1
    "if yieldBalance gets very high, decide to stop maintaining lrs"
    (Model
        (tick ?t))
    (agent-with-lrs-rule (agent ?obj) (ruleNo 1))
    (agent
        (ID ?id)
        (maintainLRS TRUE)
        (yieldBalancePerception ?y&:(fuzzy-match ?y "very high"))
        (OBJECT ?obj))
        =>
        (?obj setMaintainLRS FALSE)
;;        (printout t ?t ": agent " ?id " decides to stop maintaining LRS." crlf)
)

(defrule start-maintain-lrs-2
    "if yieldBalance gets very low, decide to maintain lrs"
    (Model
        (tick ?t))
    (agent-with-lrs-rule (agent ?obj) (ruleNo 2))
    (agent
        (ID ?id)
        (maintainLRS FALSE)
        (yieldBalancePerception ?y&:(fuzzy-match ?y "very low"))
        (OBJECT ?obj))
        =>
        (call ?obj setMaintainLRS TRUE)
        (printout t ?t ": agent " ?id " decides to maintain LRS." crlf)
)

(defrule stop-maintain-lrs-2
    "if yieldBalance gets high, decide to stop maintaining lrs"
    (Model
        (tick ?t))
    (agent-with-lrs-rule (agent ?obj) (ruleNo 2))
    (agent
        (ID ?id)
        (maintainLRS TRUE)
        (yieldBalancePerception ?y&:(fuzzy-match ?y "high"))
        (OBJECT ?obj))
        =>
        (call ?obj setMaintainLRS FALSE)
        (printout t ?t ": agent " ?id " decides to stop maintaining LRS." crlf)
)


(defrule start-maintain-lrs-3
    "if LRS condition is bad, decide to start maintaining lrs"
    (Model
        (tick ?t))
    (agent-with-lrs-rule (agent ?obj) (ruleNo 3))
    (agent
        (ID ?id)
        (maintainLRS FALSE)
        (currentLRSCond "bad")
;;        (lastMaintenanceTick ?lmt&:(= (- ?t ?lmt) 12))
        (OBJECT ?obj))
        =>
        (call ?obj setMaintainLRS TRUE)
;;        (printout t ?t ": agent " ?id " decides to maintain LRS because its condition is bad." crlf)
)

(defrule stop-maintain-lrs-3
    "if LRS condition is ok, decide to stop maintaining lrs"
    (Model
        (tick ?t))
    (agent-with-lrs-rule (agent ?obj) (ruleNo 3))
    (agent
        (ID ?id)
        (maintainLRS TRUE)
        (currentLRSCond "good")
        (OBJECT ?obj))
        =>
        (call ?obj setMaintainLRS FALSE)
;;        (printout t ?t ": agent " ?id " decides to stop maintaining LRS because LRS condition is good." crlf)
)

(defrule start-maintain-lrs-4
    "if LRS is not maintained, decide to start maintaining lrs"
    (Model
        (tick ?t))
    (agent-with-lrs-rule (agent ?obj) (ruleNo 4))
    (agent
        (ID ?id)
        (maintainLRS FALSE)
        (OBJECT ?obj))
        =>
        (call ?obj setMaintainLRS TRUE)
)



(defrule observe-wheather
    ""
    (Model
        (month 1))
    (Space
        (climate ?c))
    =>
    (bind ?w (get-member (get-member ?c currentWeather) raintype))
    (printout t "jess: climate is " ?w crlf))

