package de.uni_kassel.usf.LandReclamationJess;

import java.util.ArrayList;

public class LandParcel {

    SandboxAgent owner;

    double LRSCond = 0;
    double totalLRSCond = 0;

    double lastLRSMaintenanceTick = 0;

    double LRSMaintenanceTime = 36; //months of LRS maintenance required to get full protection
    double degradationOffset = 12;
    double LRSMaintenanceCost; //monthly cost for LRS maintenance

    double totalArea = 0;
    
    double cropLevel = 0;
    double plantingCost; //cost for yearly planting, investment in field
    
    public double calcAvgLRSCond(ArrayList<SandboxNode> nodeList){
        double sum = LRSCond, num = 1;
        for (SandboxNode n: nodeList){
            num++;
            sum+=n.getAgent().getLp().getLRSCond();
        }
        return sum/num;
    }
    
    public double calcMinLRSCondUpstream(ArrayList<SandboxNode> nodeList){
        double min, upstreamLRSCond;
        if (nodeList.isEmpty())
            min = LRSCond;
        else {
            min = 1.0;
            for (SandboxNode n: nodeList){
                upstreamLRSCond = Math.min(n.getAgent().getLp().getLRSCond(),n.getAgent().getLp().calcMinLRSCondUpstream(n.getToNodes()));
                if (upstreamLRSCond<min){
                    min = upstreamLRSCond;
                }
            }
        }
        return min;
    }
    
    public double calcTotalLRSCond(){
        double upstreamMin = calcMinLRSCondUpstream(owner.getDependencyNode().getToNodes());
        if (LRSCond==0)
            return (LRSCond+upstreamMin)/2.0;
        else
            return Math.min(LRSCond, upstreamMin);
    }
    
    public double calcYield(double propMaxLoss){
        double avgLRSCond;
//        avgLRSCond = LRSCond;
        //include condition of LRS on neighboring LP's
//        avgLRSCond = calcAvgLRSCond(owner.getDependencyNode().getToNodes());
        totalLRSCond = calcTotalLRSCond();
        return totalArea*(1.0-(1.0-totalLRSCond)*propMaxLoss);
    }
    
    public LandParcel(SandboxAgent owner, int lpSize) {
        this.owner = owner;
        totalArea = Math.pow(lpSize, 2);
        plantingCost = 0.90*totalArea;
//        plantingCost = 5*Math.sqrt(totalArea);

        LRSMaintenanceCost = (calcYield(0)-plantingCost)/12.0*(degradationOffset+1);
//        LRSMaintenanceCost = (calcYield(0)-plantingCost)/12.0*(LRSMaintenanceTime+degradationOffset)/LRSMaintenanceTime;
//        LRSMaintenanceCost = (LRSMaintenanceTime+degradationOffset)/12.0*0.1/LRSMaintenanceTime*totalArea;
//        LRSMaintenanceCost = 4*5/(LRSMaintenanceTime+degradationOffset)*Math.sqrt(totalArea);
    }

    public void plantCrop(){
        cropLevel = 1;
    }
    
    public void harvestCrop(){
        cropLevel = 0;
    }
    
    public void maintainLRS(double tick) {
        LRSCond = LRSCurveNext(LRSCond);
        lastLRSMaintenanceTick = tick;
    }

    public void degradeLRS(double tick) {
        if (tick - lastLRSMaintenanceTick > degradationOffset){
            LRSCond = LRSCurvePrev(LRSCond);
        }
    }

    private double LRSCurveY(double x) {
        if (x < 1)
            return (0);
        else if (x >= LRSMaintenanceTime)
            return (1);
        else
//            return x / (x + 1);
            return x / LRSMaintenanceTime;
    }

    private double LRSCurveX(double y) {
        if (y >= 1)
            return (LRSMaintenanceTime);
        else
//            return 1 * y / (1 - y);
            return LRSMaintenanceTime * y;
    }

    public double LRSCurveNext(double y) {
        double x = LRSCurveX(y);
        return (LRSCurveY(x + 1));
    }

    public double LRSCurvePrev(double y) {
        double x = LRSCurveX(y);
        return (LRSCurveY(x - 1));
    }

    /**
     * @return Returns the totalArea.
     */
    public double getTotalArea() {
        return totalArea;
    }

    /**
     * @param totalArea The totalArea to set.
     */
    public void setTotalArea(double totalArea) {
        this.totalArea = totalArea;
    }

    /**
     * @return Returns the lRSCond.
     */
    public double getLRSCond() {
        return LRSCond;
    }

    /**
     * @param cond The lRSCond to set.
     */
    public void setLRSCond(double cond) {
        LRSCond = cond;
    }

    /**
     * @return the plantingCost
     */
    public double getPlantingCost() {
        return plantingCost;
    }

    /**
     * @return the lRSMaintenanceCost
     */
    public double getLRSMaintenanceCost() {
        return LRSMaintenanceCost;
    }

    /**
     * @return the cropLevel
     */
    public double getCropLevel() {
        return cropLevel;
    }

    /**
     * @param cropLevel the cropLevel to set
     */
    public void setCropLevel(double cropLevel) {
        this.cropLevel = cropLevel;
    }

    /**
     * @return the lRSMaintenanceTime
     */
    public double getLRSMaintenanceTime() {
        return LRSMaintenanceTime;
    }

    /**
     * @return the degradationOffset
     */
    public double getDegradationOffset() {
        return degradationOffset;
    }

    /**
     * @return the totalLRSCond
     */
    public double getTotalLRSCond() {
        return totalLRSCond;
    }

    /**
     * @param totalLRSCond the totalLRSCond to set
     */
    public void setTotalLRSCond(double totalLRSCond) {
        this.totalLRSCond = totalLRSCond;
    }

}
