package de.uni_kassel.usf.LandReclamationJess;


import uchicago.src.collection.RangeMap;
import uchicago.src.reflector.ListPropertyDescriptor;
import uchicago.src.sim.analysis.Histogram;
import uchicago.src.sim.analysis.NetSequenceGraph;
import uchicago.src.sim.analysis.OpenSequenceGraph;
import uchicago.src.sim.analysis.PlotModel;
import uchicago.src.sim.analysis.Sequence;
import uchicago.src.sim.engine.BasicAction;
import uchicago.src.sim.engine.Controller;
import uchicago.src.sim.engine.Schedule;
import uchicago.src.sim.engine.SimModelImpl;
import uchicago.src.sim.gui.*;
import uchicago.src.sim.space.Object2DTorus;
import uchicago.src.sim.util.Random;
import uchicago.src.sim.util.SimUtilities;
import uchicago.src.sim.network.AdjacencyBitMatrix;
import uchicago.src.sim.network.AdjacencyByteMatrix;
import uchicago.src.sim.network.NetUtilities;
import uchicago.src.sim.network.NetworkConstants;
import uchicago.src.sim.network.NetworkConvertor;
import uchicago.src.sim.network.NetworkRecorder;
import uchicago.src.sim.network.Node;
import uchicago.src.sim.network.NetworkFactory;

import java.awt.*;
import java.io.*;
import java.lang.reflect.Array;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashSet;
import java.util.List;
import java.util.Vector;

import de.bkp_online.util.logger;
import jess.*;
import java.beans.*;
import nrc.fuzzy.jess.*;



public class Model extends SimModelImpl {

    // model variables
    private int numAgents = 200;
    
    private int month;
    private int year;
    
    protected int lpMaxSize = 10;
    
    private double gain = 1;
    private double consumption = 0.1;
    private double activityThreshold = 10;
    private int activityPeriod = 1;
    private double propSubsidy = 0.1;

    protected ArrayList<SandboxAgent> agentList;
    protected ArrayList<SandboxAgent> activeAgentsList;
    protected ArrayList<SandboxNode> nodeList;
    protected ArrayList<SandboxEdge> edgeList;
    protected ArrayList<SandboxNode> activeNodesList;
    private double initCollEdgeStrength = 200;
    protected ArrayList<SandboxNode> friendNodeList;
    private int maxDegree = 100;
    private int avgDegreeAcq = 4;

    protected Climate climate;
    private double propWet = 0.2;
    private double propDry = 0.4;
    private double floodingDroughtCompensationProp = 1.0;
    private double lrsCostSubsidySeasons = 1.0;


    
    protected Space space;
    private int worldXSize = 300;
    private int worldYSize = 300;

    
    private int initialSteps = 1;
    
    private NetworkRecorder nRecorder;
    private boolean writeNet = true;
    private boolean readNet = false;
    protected String filenamePath = "/projects/caves/data/test/";
    protected String filenamePrefix = "test";

    
    private AdjacencyByteMatrix lastMatrix;


    
    private double stoppingTime = 1000;
    // implementation variables
    private Schedule schedule;
    private BasicAction initialAction;
    private RangeMap neighborMap = new RangeMap();
    private RangeMap removalMap = new RangeMap();
    

    //jess stuff
    private PropertyChangeSupport pcs = new PropertyChangeSupport(this);

    public FuzzyRete engine;
    private AgentRulebase agentRulebase;
    private double tick;


    public Model() {
    }

    // builds the model
    public void buildModel() {
        
        logger.setDebug(false);
        
        edgeList.add(new SandboxEdge());

        nRecorder = new NetworkRecorder(NetworkRecorder.DL,filenamePath+filenamePrefix+".dl",this);

        climate = new Climate(propWet,propDry);

        
        space = new Space(worldXSize, worldYSize);//propSubsidy);
        space.setClimate(climate);
        space.setModel(this);
        
        int i = 0;
        int type;
        boolean agentAdded;

        SandboxNode newNode;

        System.out.println(numAgents);

        java.util.List<SandboxNode> readNodeList = null;
        if (readNet){
	         readNodeList=
	         NetworkFactory.getNetwork("/temp/test.dl", NetworkFactory.DL,
	                 SandboxNode.class, SandboxEdge.class,
	                 NetworkFactory.BINARY);
        }
      ArrayList<SandboxNode> netNodeList =
      (ArrayList<SandboxNode>) NetworkFactory.createWattsStrogatzNetwork(numAgents, avgDegreeAcq/2 , 0.1, SandboxNode.class, SandboxEdge.class);
//      ArrayList<SandboxNode> netNodeList =
//      (ArrayList<SandboxNode>) NetworkFactory.createWattsStrogatzNetwork(numAgents/3, avgDegreeAcq/2 , 0.1, SandboxNode.class, SandboxEdge.class);
//        SandboxNode n1 = netNodeList.get(0);
//
//        netNodeList.addAll((ArrayList<SandboxNode>) NetworkFactory.createWattsStrogatzNetwork(numAgents/3, avgDegreeAcq/2 , 0.1, SandboxNode.class, SandboxEdge.class));
//        SandboxNode n2 = netNodeList.get(numAgents/3);
//        
//        netNodeList.addAll((ArrayList<SandboxNode>) NetworkFactory.createWattsStrogatzNetwork(numAgents/3, avgDegreeAcq/2 , 0.1, SandboxNode.class, SandboxEdge.class));
//        SandboxNode n3 = netNodeList.get(2*numAgents/3);
        
        HashSet toEdgeList;
        ArrayList<SandboxNode> toNodeList;
        for (SandboxNode n : netNodeList) {
            toNodeList = n.getToNodes();
            for (SandboxNode tn : toNodeList) {
                toEdgeList = n.getEdgesTo(tn);
                ((SandboxEdge)toEdgeList.toArray()[0]).setTimesUsed(1);
            }
        }
        SimUtilities.shuffle(netNodeList);
        i = 0;
        int agentsAdded = 0;
        int numInnovators = 0;
        
        ExpertiseAtom ea;
        boolean isCooperative;

        do {
            isCooperative = false;
   
            SandboxAgent agent = new SandboxAgent(maxDegree,initCollEdgeStrength);

//            agent.setLandParcelSize(Random.uniform.nextIntFromTo(2,lpMaxSize));
//          agent.setLandParcelSize(agentsAdded+2);
            agent.setLandParcelSize(5);
            agentAdded = space.addAgent(agent);

            if (agentAdded) {
                agentsAdded++;
                
                agent.initPerceptions();
                System.out.println("agent " + agent.getID() + " total area " + agent.getLp().getTotalArea()+ " yield balance " + agent.getYieldBalance());
                agent.setModel(this);
                
                if (space.calcQuadrant(agent.getY())==0){
                    agent.setLRSRule(4);
                    agent.getLp().setLRSCond(1.0);
                }
                else if (space.calcQuadrant(agent.getY())==1){
                    agent.setLRSRule(0);
                    agent.getLp().setLRSCond(0.0);
                }
                else  if (space.calcQuadrant(agent.getY())==2){
                    agent.setLRSRule(0);
                    agent.getLp().setLRSCond(0.0);
                }
                else  if (space.calcQuadrant(agent.getY())==3){
                    agent.setLRSRule(0);
                    agent.getLp().setLRSCond(0.0);
                }
/*
 * this section will use a predefined sw network
 */                
                newNode = (SandboxNode) (netNodeList.get(i));
                newNode.init(0,0);
                newNode.setAgent(agent);
/*
 * ----------------------------------------------
 */             
/*
 * this initialises an unconnected node
 */
//                newNode = new SandboxNode(0,0,agent,100,100);
/*
 * ----------------------------------------------
 */
                newNode.setMaxDegree(maxDegree);
                newNode.setSemantics(SandboxNode.acquaintaincesSemantics);
                newNode.resetEdgeStrenghts();
                nodeList.add(agent.getCollaborationNode());
//                nodeList.add(agent.getDependencyNode());
                activeNodesList.add(agent.getCollaborationNode());
                agent.setAcquaintancesNode(newNode);
                friendNodeList.add(agent.getAcquaintancesNode());
                // System.out.println(agent.getNode().getOutDegree());
                agentList.add(agent);
                activeAgentsList.add(agent);
            }
        } while (agentAdded && ++i < numAgents);
        
        space.generateDependencies();

        System.out.println("No of agents: "+agentsAdded);
        if (readNet && agentsAdded<numAgents){
            for(i=agentsAdded-1;i<readNodeList.size();i++){
                SandboxNode netNode = readNodeList.get(i);
                for(SandboxNode n : nodeList){
                    n.removeEdgesTo(netNode);
                    n.removeEdgesFrom(netNode);
                }
            }
        }
        
        //jess stuff
        setTick(getTickCount());


//        engine = new Rete();
        engine = new FuzzyRete();
        /*
         * setup agent decision rules, reads the clp file into the rete engine
         */
        agentRulebase = new AgentRulebase(this);

        /*
         * define the model object and the agent objects as shadow facts
         */
        try {
            engine.definstance("Model", this, true);
            engine.definstance("Space", space, true);
            for (SandboxAgent a:agentList){
                engine.definstance("agent", a, true);

            }
        } catch (JessException je) {
            System.err.println("An error occurred at line "
                    + je.getLineNumber());
            System.err.println("caused by " + je.getCause());
            System.err.println("Message: " + je.getDetail());
            
            je.printStackTrace();
        }


    }
    

    public void distributeCollaborators(){
//        int numCooperators=0;
//        RangeMap map = new RangeMap();
//        int degree;
//        int sumDegree = 0;
//        for (SandboxAgent a:agentList){
////            degree = a.getAcquaintancesNode().getInDegree();
//            degree = 1;
////            System.out.println(degree);
//            if (degree>0)
//                map.put(sumDegree,a);
//            sumDegree+=degree;
//        }
//        SandboxAgent a;
//        int cnt = 0;
//        do {
//            a = (SandboxAgent) (map.get(Random.uniform.nextIntFromTo(0,sumDegree)));
//            if (!a.isCooperative){
//                a.isCooperative = true;
//                cnt++;
//            }
//        }while(cnt<numAgents*propCooperators);
    }
        
    public void initialAction() {

    }
    
    public void saveNodeList(java.util.List<SandboxNode> aNodeList, String name){
        nRecorder.record(aNodeList, name, NetworkRecorder.BINARY);
        nRecorder.write();        
    }
    
    public void undertaker(){
        
        ArrayList<SandboxAgent> deadAgents = new ArrayList<SandboxAgent>();
        for (SandboxAgent a : activeAgentsList) {
            if (!a.isAlive()){
                a.leaveAcquaintaincesNetwork();
                a.leaveCollaborationNetwork();
                deadAgents.add(a);
            }
        }
        for (SandboxAgent a : deadAgents) {
            activeAgentsList.remove(a);
            activeNodesList.remove(a.getCollaborationNode());
            space.removeAgentAt(a.getX(),a.getY());
        }
    }
    
    public double stAvgEdgeAge;
    public double stAvgEdgeUsage;
    public double stNumEdgesAdded;
    public double stAvgOutDegree;
    public double stAvgInDegree;
    
    public double[] stAvgYieldLPS = new double[lpMaxSize];
    public double[] stAvgLRSCondLPS = new double[lpMaxSize];
    public double[] stAvgYieldQ = new double[Space.numQuads];
    public double[] stAvgYieldBQ = new double[Space.numQuads];
    public double[] stAvgLRSCondQ = new double[Space.numQuads];
    public int numRules = 5;
    public double[] stNumAgentsR = new double[numRules];
    public double[] stAvgYieldR = new double[numRules];
    public double stAvgLRSCond;
    public double stAvgTotalLRSCond;
    
    public double stSubsReceived;
    public double stCompReceived;
    public double stAvgHarvest;
    public double stNumActiveFarmers;
       
    public void computeStats(){
        stAvgEdgeAge = 0;
        stAvgEdgeUsage = 0;
        stNumEdgesAdded = 0;
        stAvgOutDegree = 0;
        stAvgInDegree = 0;
        
        stAvgLRSCond = 0;
        stAvgTotalLRSCond = 0;
        
        stSubsReceived = 0;
        stCompReceived = 0;
        stAvgHarvest = 0;
        stNumActiveFarmers = 0;
        
        int[] numAgentsLp = new int[lpMaxSize];
        for (int i=0;i<lpMaxSize;i++){
            numAgentsLp[i] = 0;
            stAvgYieldLPS[i] = 0;
            stAvgLRSCondLPS[i] = 0;
        }
        int[] numAgentsQ = new int[Space.numQuads];
        for (int i=0;i<Space.numQuads;i++){
            numAgentsQ[i] = 0;
            stAvgYieldQ[i] = 0;
            stAvgYieldBQ[i] = 0;
            stAvgLRSCondQ[i] = 0;
        }
        int[] numAgentsR = new int[numRules];
        for (int i=0;i<numRules;i++){
            numAgentsR[i] = 0;
            stNumAgentsR[i] = 0;
            stAvgYieldR[i] = 0;
        }
        
        double numAgents = 0;
        ArrayList<SandboxEdge> oe;
        double ne,sum1e,sum2e;
        for (SandboxAgent a : agentList) {
            if (a.isAlive()){
                stNumEdgesAdded+= a.getCollEdgesAdded();
                stAvgOutDegree+=a.getCollaborationNode().getOutDegree();
                stAvgInDegree+=a.getCollaborationNode().getInDegree();
                oe = a.getCollaborationNode().getOutEdges();
                ne = 0;
                sum1e = 0;sum2e = 0;
                for (SandboxEdge e: oe){
                    sum1e+=e.getAge();
                    sum2e+=e.getTimesUsed();
                    ne++;
                }
                if (ne>0){
                    stAvgEdgeAge+=sum1e/ne;
                    stAvgEdgeUsage+=sum2e/ne;
                }
                
                stAvgYieldLPS[a.getLandParcelSize()-1]+=a.getYieldBalance();
                stAvgLRSCondLPS[a.getLandParcelSize()-1]+=a.getLp().getLRSCond();
                numAgentsLp[a.getLandParcelSize()-1]++;
                
                stAvgYieldQ[space.calcQuadrant(a.getY())]+=a.getLastYield();
                stAvgYieldBQ[space.calcQuadrant(a.getY())]+=a.getYieldBalance();
                stAvgLRSCondQ[space.calcQuadrant(a.getY())]+=a.getLp().getLRSCond();
                numAgentsQ[space.calcQuadrant(a.getY())]++;
                
                stNumAgentsR[a.getLRSRule()]+=a.getRatingMe();
//                stAvgYieldR[a.getLRSRule()]+=a.getLastYield();
                stAvgYieldR[a.getLRSRule()]+=a.getYieldBalance();
                numAgentsR[a.getLRSRule()]++;
                
                stAvgLRSCond+=a.getLp().getLRSCond();
                stAvgTotalLRSCond+=a.getLp().getTotalLRSCond();
                
                stSubsReceived+=a.getLrsCostSubsidyReceived();
                stCompReceived+=a.getFloodingDroughtCompensationReceived();
                stAvgHarvest+=a.getLastYield()-a.getFloodingDroughtCompensationReceived();
                if (a.getYieldBalance()>=a.getLp().getPlantingCost())
                    stNumActiveFarmers++;
                
                numAgents++;
            }
            else System.out.println("EEEERRRRRR");
        }
        stAvgOutDegree/=numAgents;
        stAvgInDegree/=numAgents;
        stAvgEdgeAge/=numAgents;
        stAvgEdgeUsage/=numAgents;
        
        stAvgLRSCond/=numAgents;
        stAvgTotalLRSCond/=numAgents;
        
        stSubsReceived/=numAgents;
        stCompReceived/=numAgents;
        stAvgHarvest/=numAgents;
        
        for (int i=0;i<lpMaxSize;i++){
            numAgentsLp[i] = (numAgentsLp[i]==0 ? 1: numAgentsLp[i]);
            stAvgYieldLPS[i]/=numAgentsLp[i];
            stAvgLRSCondLPS[i]/=numAgentsLp[i];
        }

        for (int i=0;i<Space.numQuads;i++){
            stAvgYieldQ[i]/=numAgentsQ[i];
            stAvgYieldBQ[i]/=numAgentsQ[i];
            stAvgLRSCondQ[i]/=numAgentsQ[i];
        }

        for (int i=0;i<numRules;i++){
//            numAgentsR[i] = (numAgentsR[i]==0 ? 1: numAgentsR[i]);
//          stAvgRatingR[i]/=numAgentsR[i];
//            stNumAgentsR[i]=numAgentsR[i];
            
            stAvgYieldR[i]/=numAgentsR[i];
            stNumAgentsR[i]=(numAgentsR[i]==0 ? stAvgYieldR[i]: numAgentsR[i]);
        }

        
    }
    
    public double computeNetworkChange(){
        double total = 0;
        AdjacencyByteMatrix presentMatrix = ((AdjacencyByteMatrix)(NetworkConvertor.nodesToMatrices(nodeList,NetworkConstants.SMALL)).get(0));
        for (int i=0;i<presentMatrix.rows();i++){
            for (int j=0;j<presentMatrix.rows();j++){
                total+=Math.abs(lastMatrix.get(i,j)-presentMatrix.get(i,j));
            }            
        }
        lastMatrix=presentMatrix;
        return total/presentMatrix.rows();
    }
    
    public double computeNetworkChange2(){
        double total = 0;
        try{
            AdjacencyByteMatrix presentMatrix = ((AdjacencyByteMatrix)(NetworkConvertor.nodesToMatrices(nodeList,NetworkConstants.SMALL)).get(0));
            for (int i=0;i<presentMatrix.rows();i++){
                for (int j=0;j<i;j++){
                    if ((lastMatrix.get(i,j)==0 && presentMatrix.get(i,j)!=0) || (lastMatrix.get(i,j)!=0 && presentMatrix.get(i,j)==0)){
                        total++;
                    }
                }            
            }
            lastMatrix=presentMatrix;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return total;///presentMatrix.rows();
    }
    

    public double computeNumEdgesAdded() {
        double total = 0;
        double numAlive = 0;
        for (SandboxAgent a : agentList) {
            if (a.isAlive()){
                total += a.getCollEdgesAdded();
                numAlive++;
            }
        }
        
        return total;// / numAlive;
      }

    public double computeAvgNumEdgesRemoved() {
        double total = 0;
        double numAlive = 0;
        for (SandboxAgent a : agentList) {
            if (a.isAlive()){
                total += a.getCollaborationNode().getNumEdgesRemoved();
                numAlive++;
            }
        }
        
        return total / numAlive / 2;
      }

    public double computePropAlive() {
        double total = 0;
        for (SandboxAgent a : agentList) {
            if (a.isAlive()) total ++;
        }
        
        return total / agentList.size();
      }

    public double computeAvgEdgeStrength() {
        double total = 0,numEdges = 0;
        for (SandboxNode n : nodeList) {
            ArrayList<SandboxEdge> outE = new ArrayList(n.getOutEdges());
            numEdges+=outE.size();
            for (SandboxEdge e : outE)
                total += e.getStrength();
        }
        if (numEdges==0)
            return 0;
        else            
            return total / numEdges / nodeList.size();
    }    
 
    public double computeAvgEdgeAge() {
        double sum = 0;
        double n = 0;
        for (SandboxAgent a : agentList) {
            ArrayList<SandboxEdge> oe = a.getCollaborationNode().getOutEdges();
            for (SandboxEdge e: oe){
                sum+=e.getAge();
                n++;
            }
        }
        
        return sum / n;
      }
    
    public double computeAvgEdgeUsage() {
        double sum = 0;
        double n = 0;
        for (SandboxAgent a : agentList) {
            ArrayList<SandboxEdge> oe = a.getCollaborationNode().getOutEdges();
            for (SandboxEdge e: oe){
                sum+=e.getTimesUsed();
                n++;
            }
        }
        
        return sum / n;
      }
    
    
    protected double numComponents;
    
    public void computeComponentsStats(ArrayList<SandboxNode> nodeList){
        int compNo;
        ArrayList<ArrayList<SandboxNode>> compNodeLists;
        compNodeLists = NetUtilities.getComponents(nodeList);
        numComponents = 0;
        for (ArrayList<SandboxNode> compNodes : compNodeLists){
            if (compNodes.size()>1){
                numComponents++;
                compNo = (int) numComponents;
            } else{
                compNo = 0;
            }
            for (SandboxNode n:compNodes){
                n.getAgent().setComponentID(compNo);
            }
        }
        
    }
    
    public void mainAction() {
        SimUtilities.shuffle(agentList);

        setTick(getTickCount());
        setMonth(((int)this.getTickCount()-1) % 12 + 1);
        setYear(((int)this.getTickCount()-1) / 12 + 1);
        
        if (month == 1){
            System.out.println("year is " + year + ", month is " + month
                    + ", climate is "
                    + space.getClimate().currentWeather.getRaintype());
            for (SandboxAgent a : agentList) {
                a.memoriseYieldBalance();
            }                
            for (SandboxAgent a : agentList) {
                a.resetStats();
            }

        }
        
        space.step();

        for (SandboxAgent a : agentList)
            if (a.isMaintainLRS()){
                a.doLRSMaintenance();
            }

        
        if (month == 5){
            for (SandboxAgent a : agentList)
                a.plantCrop();
        }
        
        if (month == 10){
            for (SandboxAgent a : agentList)
                a.harvestCrop();
        }        
        for (SandboxAgent a : agentList) {
            
            a.perceiveCurrentWeather();
            
            a.perceiveCurrentLRSCond();
            
            a.perceiveYieldBalance();

//            if (Random.uniform.nextDouble()<0.1){
//                a.setEstimationDue(true);
//            }
//            else{
//                a.setEstimationDue(false);
//            }
            if (Math.random()<0.1){
                a.setEstimationDue(true);
            }
            else{
                a.setEstimationDue(false);
            }

        }

        
        /*
         * run the jess engine, changed properties of the agent objects
         * will mark the agent shadow facts as new facts
         */
        try {
            engine.run();
        } catch (JessException je) {
            System.err.println("An error occurred at line "
                    + je.getLineNumber());
            System.err.println("caused by " + je.getCause());
            System.err.println("Message: " + je.getDetail());
            je.printStackTrace();
        }
        
        
//        for (SandboxAgent a : agentList) {
//                a.step();
//        }

        if (month==12){
        }
        
        /*
         * do clean-up after each timestep
         */
        for (SandboxAgent a : agentList) {
            a.postStep();
        }

//        undertaker();

//        computeComponentsStats(nodeList);

        // space.distributeResources();
        // System.out.println("Agent Activity:" + space.getAgentActivity());
        // if (space.getAgentActivity()==0)stop();
    }

    public void removeInitialAction() {
        schedule.removeAction(initialAction);
    }

    protected void buildSchedule() {
//        initialAction = schedule.scheduleActionAt(1, this, "initialAction");
//        schedule.scheduleActionAt(initialSteps, this, "removeInitialAction",
//                Schedule.LAST);
//      schedule.scheduleActionBeginning(initialSteps + 1, this, "mainAction");
      schedule.scheduleActionBeginning(1, this, "mainAction");

//        schedule.scheduleActionBeginning(200, new BasicAction() {
//            public void execute() {
//                if (writeNet)
//                    saveNodeList(nodeList,"Sup");
//            }
//        });
        schedule.scheduleActionAt(this.stoppingTime, new BasicAction() {
            public void execute() {
                stop();
            }
        });
        schedule.scheduleActionAtEnd(new BasicAction() {
            public void execute() {

//                Comparator comp = new Comparator() { public int compare(Object
//                        o1, Object o2) { SandboxAgent a1 = (SandboxAgent) o1;
//                        SandboxAgent a2 = (SandboxAgent) o2; return
//                        (a1.getCollaborationNode().getInDegree()>a2.getCollaborationNode().getInDegree() ? 1 :
//                        (a1.getCollaborationNode().getInDegree() < a2.getCollaborationNode().getInDegree() ? -1 : 0));}};

                    Comparator comp = new Comparator() { public int compare(Object
                            o1, Object o2) { SandboxAgent a1 = (SandboxAgent) o1;
                            SandboxAgent a2 = (SandboxAgent) o2; return
                            (a1.getYieldBalance()>a2.getYieldBalance() ? 1 :
                            (a1.getYieldBalance() < a2.getYieldBalance() ? -1 : 0));}};

                Collections.sort(agentList,comp);
                for (SandboxAgent a : agentList){
//                    if (a.getCollaborationNode().getOutDegree()>0)
                        System.out.println(a.toString());
                }

                System.out.println("Number of agents:"+activeAgentsList.size());

                showComponentsStatString(nodeList);
                
//                double sum1 = 0,cnt1 = 0,sum2 = 0,cnt2 = 0;
//                for (SandboxAgent a:agentList){
//                    if (a.cooperationProb==1){
//                        sum1+=calcAvgRating(a);
//                        cnt1++;
//                    }else{
//                        sum2+=calcAvgRating(a);
//                        cnt2++;                       
//                    }
//                }
//                java.text.DecimalFormat df = new java.text.DecimalFormat();
//                
//                
//                System.out.println("COOP rating: "+df.format(sum1/(cnt1==0 ? 1:cnt1)));
//                System.out.println("NONCOOP rating: "+df.format(sum2/(cnt2==0 ? 1:cnt2)));

                
//                System.out.println("Acquaintainces network:");
//                
//                showComponentsStatString(friendNodeList);
                
                
                if (writeNet){
                    saveAllEdgeStats(nodeList,filenamePrefix+"Coll");
                    saveNodeListPajek(nodeList,filenamePrefix+"Coll");
                    saveNodeListPajek(friendNodeList,filenamePrefix+"Acqu");
//                    ArrayList<SandboxNode> collActionNet = new ArrayList<SandboxNode>();;
//                    for (SandboxAgent a:agentList){
//                        collActionNet.add(a.getCollActionNode());
//                    }
//                    saveNodeListPajek(collActionNet,"CollAction");
                }
                System.out.println(this.getClass() + " finished!");
            }
        });
    }

    public void saveNodeListPajek(ArrayList<SandboxNode> nList,String fnPrefix) {
        PrintWriter netRecorder;
        PrintWriter cluRecorder;

        try {
            netRecorder = new PrintWriter(new BufferedWriter(new FileWriter(
                    filenamePath + fnPrefix + ".net")));

            SandboxAgent a;
            netRecorder.println("*Vertices " + nList.size());
            for (SandboxNode n : nList) {
                a = n.getAgent();
                netRecorder.println(nList.indexOf(n) + 1 + " \""
                        + a.toStringShort() + "\" 1 1");
            }
            netRecorder.println("*Arcs");
            ArrayList<SandboxNode> toNodeList;
            HashSet toEdgeList;
            for (SandboxNode n : nList) {
                toNodeList = n.getToNodes();
                for (SandboxNode tn : toNodeList) {
                    toEdgeList = n.getEdgesTo(tn);
                    netRecorder.println((nList.indexOf(n) + 1) + " "
//                            + (nList.indexOf(tn) + 1) + " 1");
                    + (nList.indexOf(tn) + 1) + " " +((SandboxEdge)toEdgeList.toArray()[0]).getTimesUsed());
                }
            }
            netRecorder.close();
/*
            cluRecorder = new PrintWriter(new BufferedWriter(new FileWriter(
                    filenamePath + fnPrefix
                            + "_types.clu")));
            cluRecorder.println("*Vertices "+nList.size());
            for (SandboxNode n : nList){
                cluRecorder.println(n.getAgent().getType());
            }
            cluRecorder.close();
*/
        
        } catch (IOException e) {
            e.printStackTrace();
        }

    }


    protected void saveEdgeStats(ArrayList<SandboxNode> nList, PrintWriter pw){
        ArrayList<SandboxNode> toNodeList;
        HashSet toEdgeList;
        for (SandboxNode n : nList) {
            toNodeList = n.getToNodes();
            for (SandboxNode tn : toNodeList) {
                toEdgeList = n.getEdgesTo(tn);
                pw.println(this.getTickCount()
                        + ","+ (nList.indexOf(n) + 1)
                        + ","+ (nList.indexOf(tn) + 1)
                        + ","+ ((SandboxEdge)toEdgeList.toArray()[0]).getTimesUsed()
                        + ","+ ((SandboxEdge)toEdgeList.toArray()[0]).getAge()
                        );
            }
        }
    }
    
    public void saveAllEdgeStats(ArrayList<SandboxNode> nList,String fnPrefix) {
        PrintWriter netRecorder;

        try {
            netRecorder = new PrintWriter(new BufferedWriter(new FileWriter(
                    filenamePath + fnPrefix + "Edges.txt")));

            netRecorder.println("\"tick\",\"fromNode\",\"toNode\",\"timesUsed\",\"age\"");
            saveEdgeStats(nList,netRecorder);
            
            netRecorder.close();
/*
            cluRecorder = new PrintWriter(new BufferedWriter(new FileWriter(
                    filenamePath + fnPrefix
                            + "_types.clu")));
            cluRecorder.println("*Vertices "+nList.size());
            for (SandboxNode n : nList){
                cluRecorder.println(n.getAgent().getType());
            }
            cluRecorder.close();
*/
        
        } catch (IOException e) {
            e.printStackTrace();
        }

    }

    protected void saveAgentStats(ArrayList<SandboxAgent> aList, PrintWriter pw){
        for (SandboxAgent a : aList) {
            pw.println(this.getTickCount()
                    + ","+ a.getIDNum()
                    + ","+ a.getCollEdgesAdded()
                    + ","+ a.getCollaborationNode().getInDegree()
                    + ","+ a.getCollaborationNode().getOutDegree()
                    );
        }
    }
    
    protected String agentStatsCols = "\"tick\"" +
    ",\"ID\"" +
    ",\"collEdgesAdded\""+
    ",\"collInDegree\",\"collOutDegree\"" +
    ",\"numCollHelp\",\"numAcqHelp\""+
    ",\"taskAssigned\",\"taskCompleted\",\"contributedToTask\"";
    
    public void saveAllAgentStats(ArrayList<SandboxAgent> aList,String fnPrefix) {
        PrintWriter aRecorder;

        try {

            aRecorder = new PrintWriter(new BufferedWriter(new FileWriter(
                    filenamePath + fnPrefix + "Agents.txt")));

            aRecorder.println(agentStatsCols);
            
            saveAgentStats(aList,aRecorder);
            
            aRecorder.close();

        } catch (IOException e) {
            e.printStackTrace();
        }

    }

    public void showComponentsStatString(ArrayList<SandboxNode> nodeList){
        ArrayList<ArrayList<SandboxNode>> compNodeLists;
        compNodeLists = NetUtilities.getComponents(nodeList);
        double total;
        int compCnt = 0;
        for (ArrayList<SandboxNode> compNodes : compNodeLists){
            total = 0;
            if (compNodes.size()>1){
                compCnt++;
                System.out.println(NetUtilities.getStatsString(compNodes));
                for (SandboxNode n:compNodes){
                    total+=n.getAgent().getYieldBalance();
                }
                System.out.println("Avg.ResourceLevel: "+total/compNodes.size());
                System.out.println();
            }
        }
        System.out.println("Total no of comps (N>1): "+compCnt);
    }
    
    public void begin() {
        buildModel();
        buildSchedule();
    }

    public void setup() {
        Random.createUniform();
        schedule = null;
        agentList = null;
        activeAgentsList = null;
        friendNodeList = null;
        nodeList = null;
        edgeList = null;
        activeNodesList = null;

        System.gc();

        schedule = new Schedule();
        agentList = new ArrayList<SandboxAgent>();
        activeAgentsList = new ArrayList<SandboxAgent>();
        friendNodeList = new ArrayList<SandboxNode>();
        nodeList = new ArrayList<SandboxNode>();
        edgeList = new ArrayList<SandboxEdge>();
        activeNodesList = new ArrayList<SandboxNode>();
        worldXSize = 40;
        worldYSize = 120;
        
        setAvgDegreeAcq(6);
        setInitCollEdgeStrength(50);
        setMaxDegree(5);
        setNumAgents(60);
        setStoppingTime(500);
//        super.setRngSeed(1);


    }

    public String[] getInitParam() {
        String[] params = { "numAgents", "worldXSize", "worldYSIze","stoppingTime"
                ,"propWet","propDry","lrsCostSubsidySeasons","floodingDroughtCompensationProp"
//                ,"maxDegree"
                ,"avgDegreeAcq"
//                ,"initCollEdgeStrength"
//                ,"propCooperators","cliquishAgents"
//                ,"numAgentExpertises","gain","consumption","propAgentsAssignedWithTask","propInitiators","activityPeriod","propSubsidy","readNet","writeNet"
                ,"filenamePrefix","filenamePath"
//                ,"propExpertise0","activityThreshold"
                };
        return params;
    }

    public Schedule getSchedule() {
        return schedule;
    }

    public String getName() {
        return "SandboxModel";
    }

    public static void main(String[] args) {
        uchicago.src.sim.engine.SimInit init = new uchicago.src.sim.engine.SimInit();
        Model model = new Model();
        init.loadModel(model, "", false);
    }

//    private void randomMeeting() {
//        int nPairs = 10;
//    
//        for (int n = 0; n < nPairs; n++) {
//            int index = Random.uniform.nextIntFromTo(0, numAgents - 1);
//            SandboxNode iNode = agentList.get(index).getNode();
//            iNode.meetRandom(nodeList, maxDegree);
//        }
//    }

    /**
         * "...At each time-step, we choose NmR1 vertices at random, with
         * probabilites proportional to Zi(Zi-1). For each vertex chosen we randomly
         * choose on pair of its neighbots to meet, and establish a new connection
         * between them if they do not have a preexisting connection and if neither
         * of them already has the maximum number z* of connections...."
         */
        private void neighborMeeting() {
    /*        neighborMap.clear();
            double zeeSum = 0;
            for (int n = 0; n < numAgents; n++) {
                SandboxNode node = (SandboxNode) agentList.get(n);
                int degree = node.getOutDegree();
                if (degree > 1) {
                    neighborMap.put(zeeSum, node);
                    zeeSum += degree * (degree - 1);
                }
            }
    
            int nM = (int) (zeeSum / 2); //will be even cause #edges even
            int numPicks = (int) Math.round(nM * Rsub1);
    
            for (int n = 0; n < numPicks; n++) {
                double pick = Random.uniform.nextDoubleFromTo(0, zeeSum);
                SandboxNode node = (SandboxNode) neighborMap.get(pick);
    
                
                 * if (node == null) { System.out.println("pick: " + pick);
                 * System.out.println("zeeSum: " + zeeSum); neighborMap.print(); }
                 
    
                if (node.getOutDegree() > 1)
                    node.meetNeighbor(maxDegree);
            }
    */    }

    /*
         * "...At each time-step, we choose NeY vertices at random, with
         * probabilities proportional to Zi. For each vertex chosen we choose one of
         * its neighbors uniformly at random and delete the connection to that
         * neighbor...."
         */
        private void removeRandomFriendship() {
    /*
            removalMap.clear();
            double zeeSum = 0;
            int nEdges = 0;
    
            for (int i = 0; i < numAgents; i++) {
                SandboxNode node = (SandboxNode) agentList.get(i);
                int degree = node.getOutDegree();
                if (degree > 1) {
                    removalMap.put(zeeSum, node);
                    zeeSum += degree * (degree - 1);
                }
    
                nEdges += degree;
            }
    
            nEdges = nEdges / 2;
            int numPicks = (int) Math.round(nEdges * removeProb);
    
            for (int n = 0; n < numPicks; n++) {
                double pick = Random.uniform.nextDoubleFromTo(0, zeeSum);
                SandboxNode node = (SandboxNode) removalMap.get(pick);
                node.removeFriend();
            }
    */
        }

    private boolean avgDegreeOK() {
            double degreeSum = 0;
            for (int i = 0; i < numAgents; i++) {
                SandboxNode node = agentList.get(i).getAcquaintancesNode();
                int degree = node.getOutDegree();
                degreeSum += degree;
            }
            if (degreeSum / numAgents >= (double) maxDegree * 0.99) {
                return true;
            } else {
    //            System.out.println(degreeSum / numAgents);
                return false;
            }
    
        }

    
    public double calcAvgRating(SandboxAgent a){
        double sum = 0,cnt = 0;
        for (Object o:a.getCollaborationNode().getInEdges()){
            sum+=((SandboxEdge)o).getReliability();
            cnt++;
        }
        return sum/(cnt==0 ? 1: cnt);
    }
    // get and set accessor methods
    /**
     * @return Returns the stoppingTime.
     */
    public double getStoppingTime() {
        return stoppingTime;
    }

    /**
     * @param stoppingTime The stoppingTime to set.
     */
    public void setStoppingTime(double stoppingTime) {
        this.stoppingTime = stoppingTime;
    }

    /**
     * @return Returns the numAgents.
     */
    public int getNumAgents() {
        return numAgents;
    }

    /**
     * @param numAgents The numAgents to set.
     */
    public void setNumAgents(int numAgents) {
        this.numAgents = numAgents;
    }

    /**
     * @return Returns the maxDegree.
     */
    public int getMaxDegree() {
        return maxDegree;
    }

    /**
     * @param maxDegree The maxDegree to set.
     */
    public void setMaxDegree(int maxDregree) {
        this.maxDegree = maxDregree;
    }

    /**
     * @return Returns the initialSteps.
     */
    public int getInitialSteps() {
        return initialSteps;
    }

    /**
     * @param initialSteps The initialSteps to set.
     */
    public void setInitialSteps(int initialSteps) {
        this.initialSteps = initialSteps;
    }

    public int getWorldXSize() {
        return worldXSize;
    }

    public void setWorldXSize(int size) {
        worldXSize = size;
    }

    public int getWorldYSize() {
        return worldYSize;
    }

    public void setWorldYSize(int size) {
        worldYSize = size;
    }


    /**
     * @return Returns the consumption.
     */
    public double getConsumption() {
        return consumption;
    }

    /**
     * @param consumption The consumption to set.
     */
    public void setConsumption(double consumption) {
        this.consumption = consumption;
    }

    /**
     * @return Returns the gain.
     */
    public double getGain() {
        return gain;
    }

    /**
     * @param gain The gain to set.
     */
    public void setGain(double gain) {
        this.gain = gain;
    }

    /**
     * @return Returns the initCollEdgeStrength.
     */
    public double getInitCollEdgeStrength() {
        return initCollEdgeStrength;
    }

    /**
     * @param initCollEdgeStrength The initCollEdgeStrength to set.
     */
    public void setInitCollEdgeStrength(double initStrategyEdgeStrength) {
        this.initCollEdgeStrength = initStrategyEdgeStrength;
    }

    /**
     * @return Returns the activityPeriod.
     */
    public int getActivityPeriod() {
        return activityPeriod;
    }

    /**
     * @param activityPeriod The activityPeriod to set.
     */
    public void setActivityPeriod(int activityPeriod) {
        this.activityPeriod = activityPeriod;
    }

    /**
     * @return Returns the readNet.
     */
    public boolean isReadNet() {
        return readNet;
    }

    /**
     * @param readNet The readNet to set.
     */
    public void setReadNet(boolean readNet) {
        this.readNet = readNet;
    }

    /**
     * @return Returns the writeNet.
     */
    public boolean isWriteNet() {
        return writeNet;
    }

    /**
     * @param writeNet The writeNet to set.
     */
    public void setWriteNet(boolean writeNet) {
        this.writeNet = writeNet;
    }

    /**
     * @return Returns the filenamePrefix.
     */
    public String getFilenamePrefix() {
        return filenamePrefix;
    }

    /**
     * @param filenamePrefix The filenamePrefix to set.
     */
    public void setFilenamePrefix(String filenamePrefix) {
        this.filenamePrefix = filenamePrefix;
    }

    /**
     * @return Returns the activityThreshold.
     */
    public double getActivityThreshold() {
        return activityThreshold;
    }

    /**
     * @param activityThreshold The activityThreshold to set.
     */
    public void setActivityThreshold(double activityTheshold) {
        this.activityThreshold = activityTheshold;
    }

    /**
     * @return Returns the propSubsidy.
     */
    public double getPropSubsidy() {
        return propSubsidy;
    }

    /**
     * @param propSubsidy The propSubsidy to set.
     */
    public void setPropSubsidy(double percSubsidy) {
        this.propSubsidy = percSubsidy;
    }

    /**
     * @return Returns the avgDegreeAcq.
     */
    public int getAvgDegreeAcq() {
        return avgDegreeAcq;
    }

    /**
     * @param avgDegreeAcq The avgDegreeAcq to set.
     */
    public void setAvgDegreeAcq(int avgDegreeAcq) {
        this.avgDegreeAcq = avgDegreeAcq;
    }

    /**
     * @return Returns the numEdgesAdded.
     */
    public double getStNumEdgesAdded() {
        return stNumEdgesAdded;
    }

    /**
     * @return Returns the avgOutDegree.
     */
    public double getStAvgOutDegree() {
        return stAvgOutDegree;
    }

    /**
     * @return Returns the avgEdgeUsage.
     */
    public double getStAvgEdgeUsage() {
        return stAvgEdgeUsage;
    }

    /**
     * @return Returns the stAvgEdgeAge.
     */
    public double getStAvgEdgeAge() {
        return stAvgEdgeAge;
    }

    /**
     * @return Returns the edgeList.
     */
    public ArrayList<SandboxEdge> getEdgeList() {
        return edgeList;
    }

    /**
     * @return Returns the filenamePath.
     */
    public String getFilenamePath() {
        return filenamePath;
    }

    /**
     * @param filenamePath The filenamePath to set.
     */
    public void setFilenamePath(String filenamePath) {
        this.filenamePath = filenamePath;
    }

    /**
     * @return Returns the stAvgInDegree.
     */
    public double getStAvgInDegree() {
        return stAvgInDegree;
    }

    /**
     * @param stAvgInDegree The stAvgInDegree to set.
     */
    public void setStAvgInDegree(double stAvgInDegree) {
        this.stAvgInDegree = stAvgInDegree;
    }

//jess stuff
    
    public void addPropertyChangeListener(PropertyChangeListener p) {
        pcs.addPropertyChangeListener(p);
    }

    public void removePropertyChangeListener(PropertyChangeListener p) {
        pcs.removePropertyChangeListener(p);
    }

    
    public void setTick(double t) {
        double oldTick = tick;
        tick = t;
        pcs.firePropertyChange("tick", oldTick, tick);
    }

    public double getTick() {
        return tick;
    }

    /**
     * @return Returns the month.
     */
    public int getMonth() {
        return month;
    }

    /**
     * @param month The month to set.
     */
    public void setMonth(int month) {
        int old = this.month;
        this.month = month;
        pcs.firePropertyChange("month", old, month);
    }

    /**
     * @return Returns the year.
     */
    public int getYear() {
        return year;
    }

    /**
     * @param year The year to set.
     */
    public void setYear(int year) {
        int old = this.year;
        this.year = year;
        pcs.firePropertyChange("year", old, year);
    }

    /**
     * @return Returns the stAvgYieldBalance.
     */
    public double[] getStAvgYieldLPS() {
        return stAvgYieldLPS;
    }

    /**
     * @return Returns the stAvgLRSCond.
     */
    public double[] getStAvgLRSCondLPS() {
        return stAvgLRSCondLPS;
    }

    /**
     * @return the stAvgYield
     */
    public double[] getStAvgYieldQ() {
        return stAvgYieldQ;
    }

    /**
     * @param stAvgYield the stAvgYield to set
     */
    public void setStAvgYieldQ(double[] stAvgYield) {
        this.stAvgYieldQ = stAvgYield;
    }

    /**
     * @return the stAvgLRSCondQ
     */
    public double[] getStAvgLRSCondQ() {
        return stAvgLRSCondQ;
    }

    /**
     * @return the stAvgYieldBQ
     */
    public double[] getStAvgYieldBQ() {
        return stAvgYieldBQ;
    }

    /**
     * @return the stAvgRatingR
     */
    public double[] getStNumAgentsR() {
        return stNumAgentsR;
    }

    /**
     * @return the stAvgYieldR
     */
    public double[] getStAvgYieldR() {
        return stAvgYieldR;
    }

    /**
     * @return the stAvgLRSCond
     */
    public double getStAvgLRSCond() {
        return stAvgLRSCond;
    }

    /**
     * @return the propDry
     */
    public double getPropDry() {
        return propDry;
    }

    /**
     * @param propDry the propDry to set
     */
    public void setPropDry(double propDry) {
        this.propDry = propDry;
    }

    /**
     * @return the propWet
     */
    public double getPropWet() {
        return propWet;
    }

    /**
     * @param propWet the propWet to set
     */
    public void setPropWet(double propWet) {
        this.propWet = propWet;
    }

    /**
     * @return the stCompReceived
     */
    public double getStCompReceived() {
        return stCompReceived;
    }

    /**
     * @return the stSubsReceived
     */
    public double getStSubsReceived() {
        return stSubsReceived;
    }

    /**
     * @return the floodingDroughtCompensationProp
     */
    public double getFloodingDroughtCompensationProp() {
        return floodingDroughtCompensationProp;
    }

    /**
     * @param floodingDroughtCompensationProp the floodingDroughtCompensationProp to set
     */
    public void setFloodingDroughtCompensationProp(
            double floodingDroughtCompensationProp) {
        this.floodingDroughtCompensationProp = floodingDroughtCompensationProp;
    }

    /**
     * @return the lrsCostSubsidySeasons
     */
    public double getLrsCostSubsidySeasons() {
        return lrsCostSubsidySeasons;
    }

    /**
     * @param lrsCostSubsidySeasons the lrsCostSubsidySeasons to set
     */
    public void setLrsCostSubsidySeasons(double lrsCostSubsidySeasons) {
        this.lrsCostSubsidySeasons = lrsCostSubsidySeasons;
    }

    /**
     * @return the stAvgHarvest
     */
    public double getStAvgHarvest() {
        return stAvgHarvest;
    }

    /**
     * @return the stNumActiveFarmers
     */
    public double getStNumActiveFarmers() {
        return stNumActiveFarmers;
    }

    /**
     * @return the stAvgTotalLRSCond
     */
    public double getStAvgTotalLRSCond() {
        return stAvgTotalLRSCond;
    }

    


}


