package de.uni_kassel.usf.LandReclamationJess;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;

import uchicago.src.sim.analysis.LocalDataRecorder;
import uchicago.src.sim.analysis.OpenSequenceGraph;
import uchicago.src.sim.analysis.Sequence;
import uchicago.src.sim.engine.Schedule;
import uchicago.src.sim.engine.SimInit;
import uchicago.src.sim.network.NetUtilities;
import uchicago.src.sim.network.NetworkRecorder;
import uchicago.src.sim.network.NetworkConstants;
import uchicago.src.sim.engine.BasicAction;


public final class ModelBatch extends Model {

    // Batch variables
    private LocalDataRecorder agentStatsRecorder;
    private LocalDataRecorder netStatsRecorder;
    private NetworkRecorder nRecorder;


    public final void setup() {

        // Initializing the original model first
        super.setup();

        setStoppingTime(10000);  // This sets the number of iterations.

    }

    protected void buildSchedule() {
        super.buildSchedule();

        Schedule schedule = super.getSchedule();
        schedule.scheduleActionAtInterval(1, new BasicAction() {
            public void execute() {
                computeStats();
                agentStatsRecorder.record();
            }
        },Schedule.LAST);

//        schedule.scheduleActionAtInterval(1, new BasicAction() {
//            public void execute() {
//                saveAgentStats(agentList,agentRecorder);                
//            }
//        },Schedule.LAST);

        schedule.scheduleActionAtInterval(1000, new BasicAction() {
            public void execute() {
                System.out.println(getTickCount());
                saveEdgeStats(nodeList,edgeRecorder);                
            }
        },Schedule.LAST);

        schedule.scheduleActionAtInterval(10000, new BasicAction() {
            public void execute() {
                System.out.println(getTickCount());
                saveNodeListPajek(nodeList,filenamePrefix+"Coll"+getTickCount());
            }
        },Schedule.LAST);


        
        schedule.scheduleActionAtEnd(new BasicAction() {
            public void execute() {
                agentStatsRecorder.write();
//                saveEdgeStats(nodeList,edgeRecorder);                
                edgeRecorder.close();
//                saveAgentStats(agentList,agentRecorder);                
                agentRecorder.close();
            }
        });
        

    }

    private PrintWriter edgeRecorder;
    private PrintWriter agentRecorder;

    
    public final void buildModel() {
        super.buildModel();


        try {
            edgeRecorder = new PrintWriter(new BufferedWriter(new FileWriter(
                    filenamePath + super.filenamePrefix + "Edges.txt")));
            edgeRecorder.println("\"tick\",\"fromNode\",\"toNode\",\"timesUsed\",\"age\"");

            agentRecorder = new PrintWriter(new BufferedWriter(new FileWriter(
                    filenamePath + super.filenamePrefix + "Agents.txt")));

            agentRecorder.println(agentStatsCols);
        
        
        } catch (IOException e) {
            e.printStackTrace();
        }

        

        
        agentStatsRecorder = new LocalDataRecorder(filenamePath+filenamePrefix+".txt", this);
        agentStatsRecorder.createNumericDataSource("stNumAcqHelp", this, "getStNumAcqHelp");
        agentStatsRecorder.createNumericDataSource("stNumCollHelp", this, "getStNumCollHelp");
        agentStatsRecorder.createNumericDataSource("stNumTasksCompleted", this, "getStNumTasksCompleted");
        agentStatsRecorder.createNumericDataSource("stNumTasksAssigned", this, "getStNumTasksAssigned");
        agentStatsRecorder.createNumericDataSource("stAvgEdgeAge", this, "getStAvgEdgeAge");
        agentStatsRecorder.createNumericDataSource("stAvgEdgeUsage", this, "getStAvgEdgeUsage");
        agentStatsRecorder.createNumericDataSource("stNumEdgesAdded", this, "getStNumEdgesAdded");
        agentStatsRecorder.createNumericDataSource("stAvgOutDegree", this, "getStAvgOutDegree");
        agentStatsRecorder.createNumericDataSource("stAvgInDegree", this, "getStAvgInDegree");
        
//        netStatsRecorder = new LocalDataRecorder("/projects/caves/data/test/network_stats.txt", this);
//        netStatsRecorder.createNumericDataSource("ClusteringCoeff", this, "computeClusterCoeff");
//        netStatsRecorder.createNumericDataSource("Density", this, "computeComponents");
//        
//        nRecorder = new NetworkRecorder(NetworkRecorder.DL,"/projects/caves/data/test/fk_matrix.dl", this);

    }

    public String[] getInitParam() {
        String[] myParams = {};

        String[] params = new String[(super.getInitParam()).length+myParams.length];
        System.arraycopy(super.getInitParam(),0,params,0,(super.getInitParam()).length);
        System.arraycopy(myParams,0,params,(super.getInitParam()).length,myParams.length);

        return params;
    }

    public String getName() {
        return "SandboxModel";
    }

    public static void main(String[] args) {
        uchicago.src.sim.engine.SimInit init = new uchicago.src.sim.engine.SimInit();
        ModelBatch model = new ModelBatch();
        init.loadModel(model, "", false);
    }
}
