package de.uni_kassel.usf.LandReclamationJess;

import uchicago.src.reflector.ListPropertyDescriptor;
import uchicago.src.sim.analysis.Histogram;
import uchicago.src.sim.analysis.NetSequenceGraph;
import uchicago.src.sim.analysis.OpenSequenceGraph;
import uchicago.src.sim.analysis.Sequence;
import uchicago.src.sim.analysis.PlotModel;
import uchicago.src.sim.engine.BasicAction;
import uchicago.src.sim.engine.Schedule;
import uchicago.src.sim.gui.*;
import uchicago.src.sim.space.Object2DTorus;
import uchicago.src.sim.network.NetUtilities;
import uchicago.src.sim.network.NetworkRecorder;
import java.awt.*;
import java.util.*;

/**
 * GUI part of the Sandbox Model
 */
public class ModelGUI extends Model {

    // model variables

    private DisplaySurface worldSurface;
    private boolean showWorld = false;

    // implementation variables
    private String layoutType = "Fruch";

    private DisplaySurface networkSurface;
    private boolean showNet = false;
    protected ArrayList<SandboxNode> connectedNodesList;


    private AbstractGraphLayout networkLayout;
    private int networkXSize;
    private int networkYSize;

    private Histogram inDegreeDistColl;
    private Histogram outDegreeDistColl;
    private Histogram edgeAgeDistColl;    
    private Histogram degreeDistAcq;
    private Histogram typeDist;
    private Histogram compSizeDist;

    private boolean showHist = false;

    private NetSequenceGraph graph;

    private OpenSequenceGraph plotGraph;

    private OpenSequenceGraph yieldGraph;

    private OpenSequenceGraph yieldBGraph;

    private OpenSequenceGraph ruleUsageGraph;

    private OpenSequenceGraph weatherGraph;

    private OpenSequenceGraph LRSCondGraph;

    private OpenSequenceGraph incomeGraph;

    private boolean showPlot = true;

    private BasicAction initialAction;

    public ModelGUI() {
        Vector vect = new Vector();
        vect.add("Fruch");
        vect.add("KK");
        vect.add("CircleLayout");
        ListPropertyDescriptor pd = new ListPropertyDescriptor("LayoutType",
                vect);
        descriptors.put("LayoutType", pd);
    }

    // builds the model
    public void buildModel() {
        // Build the original model first
        super.buildModel();
        
        if (showHist)
            makeHistogram();
        if (showPlot)
            makePlot();
    }

    public void buildDisplay() {
        if (layoutType.equals("KK")) {
            networkLayout = new KamadaGraphLayout(nodeList, this.networkXSize, this.networkYSize, networkSurface, 10);
        } else if (layoutType.equals("Fruch")) {
            networkLayout = new FruchGraphLayout(nodeList, this.networkXSize, this.networkYSize, networkSurface,0);
        } else if (layoutType.equals("CircleLayout")) {
            networkLayout = new CircularGraphLayout(nodeList, this.networkXSize, this.networkYSize);
        }

        // these four lines hook up the graph layouts to the stop, pause, and
        // exit buttons on the toolbar. When stop, pause, or exit is clicked
        // the graph layouts will interrupt their layout as soon as possible.
        /*
         * Controller c = (Controller) getController();
         * c.addStopListener(networkLayout); c.addPauseListener(networkLayout);
         * c.addExitListener(networkLayout);
         */
        Network2DDisplay displayNetwork = new Network2DDisplay(networkLayout);

        networkSurface.addDisplayableProbeable(displayNetwork, "Collaboration network display");

        // add the display as a Zoomable. This means we can "zoom" in on
        // various parts of the network.
        networkSurface.addZoomable(displayNetwork);
        networkSurface.setBackground(java.awt.Color.white);
        addSimEventListener(networkSurface);



        ColorMap map = new ColorMap();

//        map.mapColor(0,Color.white);
//        for(int i = 0; i<space.getNumAgentExpertises(); i++){
//            map.mapColor(i,0, 1,0,(double)(i+1) /(double)(space.getNumAgentExpertises()));
//        }
//        
//        for (int i=0; i<space.getNumAgentExpertises(); i++){
//            System.out.println(map.getColor(i));
//        }
        
        ColorMap resMap = new ColorMap();

        resMap.mapColor(0,0, 0,0,0);
        resMap.mapColor(1,0,1,0,0.3);
        Value2DDisplay displayResources = 
            new Value2DDisplay(space.getCurrentResourceSpace(), resMap);

        ColorMap cmap = new ColorMap();

//      cmap.mapColor(0,0, 1,0,0.3);
        cmap.mapColor(0,0, 0,0,0);
        cmap.mapColor(1,Color.blue);
        Value2DDisplay displayChannels = new Value2DDisplay(space.getChannelSpace(),cmap);
        
        ColorMap lpMap = new ColorMap();
//		for (int i = 1; i <= agentList.size(); i++) {
//          lpMap.mapColor(i, 1, 0, 0, (double) (i + 1)
//          / (double) (agentList.size()));
//          lpMap.mapColor(i, 1, 0, 0, 1);
//		}
//        lpMap.mapColor(0, Color.white);
        lpMap.mapColor(0, 0,0,0,0);
        for (int i = 0; i < agentList.size(); i++) {
//            lpMap.mapColor((new Integer(agentList.get(i).getID())).intValue(), Color.cyan);
            lpMap.mapColor((new Integer(agentList.get(i).getID())).intValue(), 0, 1,0,0.5);
        }
		Value2DDisplay displayLandParcels = new Value2DDisplay(space.getLandParcelSpace(), lpMap);
        
        Object2DDisplay displayAgents = new Object2DDisplay(space.getCurrentAgentSpace());
        displayAgents.setObjectList(agentList);

//        worldSurface.addDisplayableProbeable(displayResources, "Land Cover");
//        worldSurface.addDisplayableProbeable(displayChannels, "Channels");
        worldSurface.addDisplayableProbeable(displayAgents, "Agents");
//        worldSurface.addDisplayableProbeable(displayLandParcels, "Ownership");

        
        worldSurface.setBackground(java.awt.Color.white);
        addSimEventListener(worldSurface);
        
        
         
    }

    /*
     * Creates a histogram of the degree distribution.
     */
    private void makeHistogram() {

        inDegreeDistColl = new Histogram("InDegree distribution coll network",50,0,50,this);

        inDegreeDistColl.createHistogramItem("InDegree distribution", activeNodesList,
                "getInDegree");
        
        outDegreeDistColl = new Histogram("OutDegree distribution coll network",50,0,50,this);

        outDegreeDistColl.createHistogramItem("OutDegree distribution", activeNodesList,
                "getOutDegree");
        
        edgeAgeDistColl = new Histogram("EdgeAge distribution coll network",100,0,1000,this);

        edgeAgeDistColl.createHistogramItem("EdgeAge distribution", edgeList,
                "getAge");
        
        degreeDistAcq = new Histogram("Degree distribution acq network",25,0,50,this);

        degreeDistAcq.createHistogramItem("Degree distribution", friendNodeList,
                "getInDegree");
        
        typeDist = new Histogram("LRS rule distribution",10,0,10,this);

        typeDist.createHistogramItem("LRS rule distribution", activeAgentsList,
                "getLRSRule");
        
        compSizeDist = new Histogram("Component size distribution",11,0,11,this);

        compSizeDist.createHistogramItem("Component sizes", activeAgentsList,
                "getComponentID");
        
    }

    /*
     * Creates a Plot of the Clustering Coefficient, the avg. density, and the
     * component count.
     */
    
    private class MSeq implements Sequence{
        public final int colNo;
        private double[] dataArray;
        public MSeq(double[] dataArray,int colNo){
            this.dataArray = dataArray; 
            this.colNo = colNo; 
        }
        public double getSValue() {
            return dataArray[colNo];
          };        
    }
    
    private void makePlot() {

        graph = new NetSequenceGraph("Network stats coll network", this,
                filenamePath + "collnet.txt", PlotModel.CSV, nodeList);
        graph.setAxisTitles("Time", "Value");
        graph.setYRange(0, 2);
        graph.setYViewPolicy(uchicago.src.sim.analysis.OpenSequenceGraph.SHOW_ALL);
//        graph.addSequence("Acq network activity", new Sequence() {
//            public double getSValue() {
//              return getStNumAcqHelp();
//            }
//            }, OpenSequenceGraph.FILLED_CIRCLE);

//        graph.addSequence("Coll network activity", new Sequence() {
//            public double getSValue() {
//                return getStNumCollHelp();
//            }
//            }, OpenSequenceGraph.FILLED_CIRCLE);


//        graph.addSequence("Coll network errors", new Sequence() {
//            public double getSValue() {
//                return getStNumCollNoHelp();
//            }
//            }, OpenSequenceGraph.FILLED_CIRCLE);

        graph.addSequence("Avg. out degree (Coll)", new Sequence() {
            public double getSValue() {
                return getStAvgOutDegree();
              }
            }
            ,OpenSequenceGraph.FILLED_CIRCLE);

//        graph.addSequence("Avg edge age", new Sequence() {
//            public double getSValue() {
//                return computeAvgEdgeAge();
//            }
//            }, OpenSequenceGraph.FILLED_CIRCLE);
//
//        graph.addSequence("Avg edge usage", new Sequence() {
//            public double getSValue() {
////                return computeAvgEdgeUsage();
//                return getStAvgEdgeUsage();
//            }
//            }, OpenSequenceGraph.FILLED_CIRCLE);

        
        plotGraph = new OpenSequenceGraph("Agent stats", this,
                filenamePath + "agents.txt", PlotModel.CSV);
        plotGraph.setAxisTitles("Time", "Statistic value");
        plotGraph.setYRange(0, 1);
        plotGraph.setYIncrement(0.1);
        plotGraph.setYViewPolicy(uchicago.src.sim.analysis.OpenSequenceGraph.SHOW_ALL);

//        plotGraph.addSequence("Support network activity", new Sequence() {
//            public double getSValue() {
//                return computeCollaborationNetworkActivity();
//            }
//        }, OpenSequenceGraph.FILLED_CIRCLE);
//
//        
//
//        plotGraph.addSequence("Avg. tasks completed", new Sequence() {
//            public double getSValue() {
//                return computeAvgTasksCompleted();
//            }
//        }, OpenSequenceGraph.FILLED_CIRCLE);

//        plotGraph.addSequence("Avg. network activity", new Sequence() {
//            public double getSValue() {
//                return computeTotalNetworkActivity();
//            }
//        }, OpenSequenceGraph.FILLED_CIRCLE);

        plotGraph.addSequence("Num edges added (Coll)", new Sequence() {
            public double getSValue() {
//                return computeNumEdgesAdded();
                return getStNumEdgesAdded();
              }
            }
            ,OpenSequenceGraph.FILLED_CIRCLE);

//        plotGraph.addSequence("Perc. tasks completed", new Sequence() {
//            public double getSValue() {
////                return computePercTasksCompleted();
//                return getStNumTasksCompleted()/getStNumTasksAssigned()*100;
//              }
//            }
//            ,OpenSequenceGraph.FILLED_CIRCLE);
        


        yieldGraph = new OpenSequenceGraph("Agent yield stats", this,
                filenamePath + "agents_yield.txt", PlotModel.CSV);
        yieldGraph.setAxisTitles("Time", "Statistic value");
        yieldGraph.setYRange(0, 10);
        yieldGraph.setYIncrement(10);
        yieldGraph.setYViewPolicy(uchicago.src.sim.analysis.OpenSequenceGraph.SHOW_ALL);

        
//        for (int i = 1; i<=lpMaxSize; i++){
//            yieldGraph.addSequence("Avg. yield balance (lpSize=" + i +")",new MSeq(getStAvgYieldBalance(),i-1)
//                    ,OpenSequenceGraph.FILLED_CIRCLE);
//        }              

        for (int i = 1; i<=Space.numQuads; i++){
            yieldGraph.addSequence("Avg. yield (quad=" + i +")",new MSeq(getStAvgYieldQ(),i-1)
                    ,OpenSequenceGraph.FILLED_CIRCLE);
        }              

        yieldBGraph = new OpenSequenceGraph("Agent yield stats", this,
                filenamePath + "agents_yield_balance.txt", PlotModel.CSV);
        yieldBGraph.setAxisTitles("Time", "Statistic value");
        yieldBGraph.setYRange(0, 10);
        yieldBGraph.setYIncrement(10);
        yieldBGraph.setYViewPolicy(uchicago.src.sim.analysis.OpenSequenceGraph.SHOW_ALL);

        for (int i = 0; i<numRules; i++){
            yieldBGraph.addSequence("Avg. yield balance (rule=" + i +")",new MSeq(getStAvgYieldR(),i)
                    ,OpenSequenceGraph.FILLED_CIRCLE);
        }              

    
        ruleUsageGraph = new OpenSequenceGraph("Rules", this,
                filenamePath + "rules.txt", PlotModel.CSV);
        ruleUsageGraph.setAxisTitles("Time", "");
        ruleUsageGraph.setYRange(0, 30);
        ruleUsageGraph.setYIncrement(1);
        ruleUsageGraph.setYViewPolicy(uchicago.src.sim.analysis.OpenSequenceGraph.SHOW_ALL);

        for (int i = 0; i<numRules; i++){
            ruleUsageGraph.addSequence("Num of agents (rule=" + i +")",new MSeq(getStNumAgentsR(),i)
                    ,OpenSequenceGraph.FILLED_CIRCLE);
        }              

        ruleUsageGraph.addSequence("Active agents", new Sequence() {
            public double getSValue() {
                return getStNumActiveFarmers();
              }
            }        ,OpenSequenceGraph.FILLED_CIRCLE);

    
        weatherGraph = new OpenSequenceGraph("Weather condition", this,
                filenamePath + "weather.txt", PlotModel.CSV);
        weatherGraph.setAxisTitles("Time", "");
        weatherGraph.setYRange(-1, 1);
        weatherGraph.setYIncrement(0.1);
        weatherGraph.setYViewPolicy(uchicago.src.sim.analysis.OpenSequenceGraph.SHOW_ALL);
        
        weatherGraph.addSequence("Weather", new Sequence() {
            public double getSValue() {
                return space.getClimate().currentWeather.getRaintype();
              }
            }
            ,OpenSequenceGraph.FILLED_TRIANGLE);

        
        LRSCondGraph = new OpenSequenceGraph("Condition of LRS", this,
                filenamePath + "LRSCond.txt", PlotModel.CSV);
        LRSCondGraph.setAxisTitles("Time", "");
        LRSCondGraph.setYRange(0, 1);
        LRSCondGraph.setYIncrement(0.1);
        LRSCondGraph.setYViewPolicy(uchicago.src.sim.analysis.OpenSequenceGraph.SHOW_ALL);
        
        LRSCondGraph.addSequence("Local", new Sequence() {
            public double getSValue() {
                return getStAvgLRSCond();
              }
            }        ,OpenSequenceGraph.FILLED_CIRCLE);

        LRSCondGraph.addSequence("Total", new Sequence() {
            public double getSValue() {
                return getStAvgTotalLRSCond();
              }
            }        ,OpenSequenceGraph.FILLED_CIRCLE);

        incomeGraph = new OpenSequenceGraph("Average income", this,
                filenamePath + "Income.txt", PlotModel.CSV);
        incomeGraph.setAxisTitles("Time", "");
        incomeGraph.setYRange(0, 10);
        incomeGraph.setYIncrement(1);
        incomeGraph.setYViewPolicy(uchicago.src.sim.analysis.OpenSequenceGraph.SHOW_ALL);
        
        incomeGraph.addSequence("Gain from harvest", new Sequence() {
            public double getSValue() {
                return getStAvgHarvest();
              }
            }        ,OpenSequenceGraph.FILLED_CIRCLE);

        incomeGraph.addSequence("LRS subsidies", new Sequence() {
            public double getSValue() {
                return getStSubsReceived();
              }
            }        ,OpenSequenceGraph.FILLED_CIRCLE);

        incomeGraph.addSequence("Compensation payments", new Sequence() {
            public double getSValue() {
                return getStCompReceived();
              }
            }        ,OpenSequenceGraph.FILLED_CIRCLE);

    }

    public void initialAction() {
        graphStuff();
//        networkLayout.updateLayout();
//        networkSurface.updateDisplay();
        worldSurface.updateDisplay();
    }

    public void plotStuff() {
        if (showHist){
//            inDegreeDistColl.step();
//            outDegreeDistColl.step();
//            degreeDistAcq.step();
            typeDist.step();
//            compSizeDist.step();
        }
        if (showPlot){
            if (getMonth()==12){
                computeStats();
                yieldGraph.step();
                yieldBGraph.step();
                ruleUsageGraph.step();
                weatherGraph.step();
                incomeGraph.step();
                LRSCondGraph.step();
            }
        }
    }
    
    public void writePlots(){
        System.out.println("writeplots");
        yieldGraph.writeToFile();
		yieldBGraph.writeToFile();
		ruleUsageGraph.writeToFile();
		weatherGraph.writeToFile();
		incomeGraph.writeToFile();
		LRSCondGraph.writeToFile();
    }
    


    public void edgeHistStuff() {
        if (showHist){
            edgeAgeDistColl.step();
        }
    }

    public void graphStuff() {
        
        if (showNet) {
//            connectedNodesList.clear();
//            connectedNodesList.addAll(nodeList);
//            networkLayout.setList(connectedNodesList);
//          ((FruchGraphLayout)networkLayout).setAnimateTransitions(false);
//            ((FruchGraphLayout)networkLayout).setPad(1);
//            networkLayout.updateLayout();
            networkSurface.updateDisplay();
        }
        if (showWorld) worldSurface.updateDisplay();
    }

    public void updateNetworkLayout() {
		networkLayout.updateLayout();
	}

    protected void buildSchedule() {
        super.buildSchedule();

        Schedule schedule = super.getSchedule();
        schedule.scheduleActionAtInterval(10, this, "edgeHistStuff", Schedule.LAST);
        schedule.scheduleActionAtInterval(12, this, "graphStuff", Schedule.LAST);
        schedule.scheduleActionAtInterval(12, this, "updateNetworkLayout", Schedule.LAST);
        schedule.scheduleActionAtInterval(1, this, "plotStuff", Schedule.LAST);
        schedule.scheduleActionAtEnd(this, "writePlots");
    }

    public void begin() {
        super.begin();
        if (showHist){
//            inDegreeDistColl.display();
//            outDegreeDistColl.display();
//            edgeAgeDistColl.display();
//            degreeDistAcq.display();
            typeDist.display();
//            compSizeDist.display();
        }
        if (showPlot) {
//            graph.display();
//            plotGraph.display();
            yieldGraph.display();
            yieldBGraph.display();
            ruleUsageGraph.display();
            weatherGraph.display();
            LRSCondGraph.display();
            incomeGraph.display();
        }
        buildDisplay();
        if (showNet)
            networkSurface.display();
        if (showWorld)
            worldSurface.display();
    }

    public void setup() {
        // Initializing the original model first
        super.setup();

        connectedNodesList = null;
        connectedNodesList = new ArrayList<SandboxNode>();

        if (networkSurface != null)
            networkSurface.dispose();
        if (inDegreeDistColl != null)
            inDegreeDistColl.dispose();
        if (outDegreeDistColl != null)
            outDegreeDistColl.dispose();
        if (edgeAgeDistColl != null)
            edgeAgeDistColl.dispose();
        if (degreeDistAcq != null)
            degreeDistAcq.dispose();
        if (typeDist != null)
            typeDist.dispose();
        if (compSizeDist != null)
            compSizeDist.dispose();
        if (graph != null)
            graph.dispose();
        if (plotGraph != null)
            plotGraph.dispose();
        if (yieldGraph != null)
            yieldGraph.dispose();
        if (yieldBGraph != null)
            yieldBGraph.dispose();
        if (weatherGraph != null)
            weatherGraph.dispose();
        if (ruleUsageGraph != null)
            ruleUsageGraph.dispose();
        if (LRSCondGraph != null)
            LRSCondGraph.dispose();
        if (incomeGraph != null)
            incomeGraph.dispose();
        if (worldSurface != null)
            worldSurface.dispose();

        networkSurface = null;
        inDegreeDistColl = null;
        outDegreeDistColl = null;
        edgeAgeDistColl = null;
        degreeDistAcq = null;
        typeDist = null;
        compSizeDist = null;
        graph = null;
        plotGraph = null;
        yieldGraph = null;
        yieldBGraph = null;
        ruleUsageGraph = null;
        weatherGraph = null;
        LRSCondGraph = null;
        incomeGraph = null;
        worldSurface = null;

        System.gc();

        networkXSize = 2000;
        networkYSize = 2000;
        //change model parameters to suitable values for gui demo run
        setAvgDegreeAcq(10);
        setInitCollEdgeStrength(50);
        setMaxDegree(5);
        setNumAgents(30);
        setStoppingTime(500);
//        setShowWorld(true);
//        setShowHist(true);
        DisplayConstants.CELL_HEIGHT = 8;
        DisplayConstants.CELL_WIDTH = 8;

//        super.setRngSeed(1);


        networkSurface = new DisplaySurface(this, "Network Surface");
        registerDisplaySurface("Main Display", networkSurface);


        worldSurface = new DisplaySurface(this, "Agent Surface");
        registerDisplaySurface("Main Display", worldSurface);

    }

    public String[] getInitParam() {
        String[] myParams = { "layoutType", "showHist", "showNet", "showWorld", "plot"};

        String[] params = new String[(super.getInitParam()).length+myParams.length];
        System.arraycopy(super.getInitParam(),0,params,0,(super.getInitParam()).length);
        System.arraycopy(myParams,0,params,(super.getInitParam()).length,myParams.length);

        return params;
    }

    public String getName() {
        return "SandboxModel";
    }

    public static void main(String[] args) {
        uchicago.src.sim.engine.SimInit init = new uchicago.src.sim.engine.SimInit();
        ModelGUI model = new ModelGUI();
        init.loadModel(model, "", false);
    }

    // get and set accessor methods
    public boolean getShowHist() {
        return showHist;
    }

    public void setShowHist(boolean val) {
        showHist = val;
    }

    public boolean getPlot() {
        return showPlot;
    }

    public void setPlot(boolean val) {
        showPlot = val;
    }

    public String getLayoutType() {
        return layoutType;
    }

    public void setLayoutType(String type) {
        layoutType = type;
    }

    /**
     * @return Returns the networkXSize.
     */
    public int getNetworkXSize() {
        return networkXSize;
    }

    /**
     * @param networkXSize The networkXSize to set.
     */
    public void setNetworkXSize(int networkXSize) {
        this.networkXSize = networkXSize;
    }

    /**
     * @return Returns the networkYSize.
     */
    public int getNetworkYSize() {
        return networkYSize;
    }

    /**
     * @param networkYSize The networkYSize to set.
     */
    public void setNetworkYSize(int networkYSize) {
        this.networkYSize = networkYSize;
    }

    /**
     * @return Returns the showNet.
     */
    public boolean isShowNet() {
        return showNet;
    }

    /**
     * @param showNet The showNet to set.
     */
    public void setShowNet(boolean showNet) {
        this.showNet = showNet;
    }

    /**
     * @return Returns the showWorld.
     */
    public boolean isShowWorld() {
        return showWorld;
    }

    /**
     * @param showWorld The showWorld to set.
     */
    public void setShowWorld(boolean showWorld) {
        this.showWorld = showWorld;
    }
}
