/*$$
 *Simple agent for testing purposes.
 *
 *$$*/
package de.uni_kassel.usf.LandReclamationJess;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;

import uchicago.src.collection.RangeMap;
import uchicago.src.sim.util.Random.*;
import java.util.*;


import uchicago.src.sim.gui.ColorMap;
import uchicago.src.sim.gui.Drawable;
import uchicago.src.sim.gui.SimGraphics;
import uchicago.src.sim.gui.DisplayConstants;

import uchicago.src.sim.util.Random;
import uchicago.src.sim.util.SimUtilities;

//for jess
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

import de.bkp_online.util.logger;

import nrc.fuzzy.*;

/**
 *
 */

public class SandboxAgent implements Drawable {

    
    private Model model;

    private SandboxNode acquaintancesNode;

    private SandboxNode collaborationNode;
    
    private SandboxNode dependencyNode;

    private int landParcelSize;
    
    private int x, y, dx, dy;

    private static int IDNumber = 0;

    private int ID;

    private Space space;

    private int strategySwitched = 0;

    private ArrayList<SandboxNode> collaborationNetworkNeighbors;
//    private Collection<SandboxNode> collaborationNetworkNeighbors;
    private int componentID;


    private ArrayList<SandboxNode> friendNetworkNeighbors;

    private int numFriendsPolled = 0;

    private int numSupportersPolled = 0;

    private int collEdgesAdded;
    private int numCollEdgesAdded = 0;

    private int acqEdgesAdded;

    /**
     * Creates this Agent with the specified type.
     */
    public SandboxAgent(int maxDegree, double initCollEdgeStrength) {
        
        logger.setDebug(false);
        
        this.collaborationNode = new SandboxNode(this,maxDegree,initCollEdgeStrength);
        this.collaborationNode.setSemantics(SandboxNode.coworkerSemantics);

        this.dependencyNode = new SandboxNode(this,maxDegree,initCollEdgeStrength);
        this.dependencyNode.setSemantics(SandboxNode.dependencySemantics);

        IDNumber++;
        ID = IDNumber;

//        initPerceptions();
        ratingMe = 1;
        ratingOthers = 1;

    }

    public void leaveAcquaintaincesNetwork(){
        for(SandboxNode n : (ArrayList<SandboxNode>)(acquaintancesNode.getToNodes())){
            this.acquaintancesNode.removeEdgesTo(n);
            n.removeEdgesFrom(this.acquaintancesNode);
        }
        for(SandboxNode n : (ArrayList<SandboxNode>)(acquaintancesNode.getFromNodes())){
            this.acquaintancesNode.removeEdgesFrom(n);
            n.removeEdgesTo(this.acquaintancesNode);
        }

    }

    public void leaveCollaborationNetwork(){
        for(SandboxNode n : (ArrayList<SandboxNode>)(collaborationNode.getToNodes())){
            this.collaborationNode.removeEdgesTo(n);
            n.removeEdgesFrom(this.collaborationNode);
        }
        for(SandboxNode n : (ArrayList<SandboxNode>)(collaborationNode.getFromNodes())){
            this.collaborationNode.removeEdgesFrom(n);
            n.removeEdgesTo(this.collaborationNode);
        }
    }

    public void perceivePhysicalEnvironment(){
        /*
         * perceive the physical environment
         */
    }
    
    public void resetStats(){
        collEdgesAdded = 0;
        acqEdgesAdded = 0;
        floodingDroughtCompensationReceived = 0.0;
        lrsCostSubsidyReceived = 0.0;
    }
    
    public void perceiveSocialEnvironment(){
        /*
         * perceive the social environments
         */
        

        collaborationNetworkNeighbors = collaborationNode.getToNodes();        
        SimUtilities.shuffle(collaborationNetworkNeighbors);

        
        friendNetworkNeighbors = acquaintancesNode.getToNodes();
        SimUtilities.shuffle(friendNetworkNeighbors);
        /*
         * reset agent based statistics
         */
        numSupportersPolled = collaborationNetworkNeighbors.size();
        numFriendsPolled = friendNetworkNeighbors.size();

    }

    public void prestep() {
        perceivePhysicalEnvironment();
        perceiveSocialEnvironment();
    }

    
    public void step() {

    }

    public void postStep(){
        collaborationNode.decayEdges();
        collaborationNode.ageEdges();
    }


    /**
     * @return Returns the acquaintancesNode.
     */
    public SandboxNode getAcquaintancesNode() {
        return acquaintancesNode;
    }

    /**
     * @param acquaintancesNode
     *            The acquaintancesNode to set.
     */
    public void setAcquaintancesNode(SandboxNode node) {
        this.acquaintancesNode = node;
    }

    // Drawable interface
    public int getX() {
        return x;
    }

    public int getY() {
        return y;
    }

    public void setXY(int newX, int newY) {
        x = newX;
        y = newY;
    }

    public void setDxDy(int newDx, int newDy) {
        dx = newDx;
        dy = newDy;
    }

    public void setDxDy(int direction) {
        switch (direction) {
        case 0: {
            setDxDy(0, 1);
            break;
        }
        case 1: {
            setDxDy(1, 1);
            break;
        }
        case 2: {
            setDxDy(1, 0);
            break;
        }
        case 3: {
            setDxDy(1, -1);
            break;
        }
        case 4: {
            setDxDy(0, -1);
            break;
        }
        case 5: {
            setDxDy(-1, -1);
            break;
        }
        case 6: {
            setDxDy(-1, 0);
            break;
        }
        case 7: {
            setDxDy(-1, 1);
            break;
        }
        default: {
            setDxDy(0, 0);
        }
        }

    }

    public void setSpace(Space sp) {
        space = sp;
    }

    public String getID() {
        return ""+ID;
    }

    public String getIDNum() {
        return ""+ID;
    }


    java.text.DecimalFormat df = new java.text.DecimalFormat();

    public String toString() {
        double sum = 0, cnt = 0;
        
//            return getID()+"-"+(isCooperative ? "C":"!C")+"-"+getExpertiseID()+" Degree:"+collaborationNode.getInDegree()+" / "+collaborationNode.getOutDegree()+" ResourceLevel:"+df.format(resourceLevel)+" Tasks:"+numTasksAssigned+" CompNo:"+componentID;
        return getID()+"-"+"Balance: " + df.format(this.yieldBalance) +"-Last: " + df.format(this.lastYield) + "-Quad: " + (space.calcQuadrant(y)+1) + "-Rule:" + LRSRule +"-Rating:" + df.format(ratingMe) + "-Degree: "+acquaintancesNode.getInDegree()+" / "+acquaintancesNode.getOutDegree();
    }
    public String toStringShort() {
        return getID();
    }

    private static final Color[] COLOR = {Color.RED, Color.CYAN, Color.BLUE, Color.MAGENTA, Color.ORANGE};

    private static ColorMap colMapLRS;
    static {
        colMapLRS = new ColorMap();
        for (int i = 0; i <= 100; i++) {
            colMapLRS.mapColor(i, 0, (float)i / 100f, 0, 0.5);
        }
    }

    private static ColorMap colMapGYR;
    static {
        colMapGYR = new ColorMap();
        colMapGYR.mapColor(0, 0, 1, 0, 0.5);
        colMapGYR.mapColor(1, Color.yellow.getRed(),Color.yellow.getGreen(),Color.yellow.getBlue(),0.5);
        colMapGYR.mapColor(2, 1, 0, 0, 0.5);
    }

    public void draw(SimGraphics g) {
//        g.drawFastRoundRect(COLOR[LUStrategy-1]);
//        if (this.isAlive()) g.drawFastRoundRect(space.agentColors.getColor(type));
//        if (this.isAlive()) g.drawFastRoundRect(space.agentColors.getColor(componentID));

    	
//        Graphics2D g2d = g.getGraphics();
//        Rectangle r = g2d.getClipBounds();
//        int x = (int) r.getMinX();
//        int y = (int) r.getMinY();
//        g.setDrawingCoordinates(x + DisplayConstants.CELL_WIDTH / 6,
//                                y + DisplayConstants.CELL_HEIGHT / 6, 0);
//        g.setDrawingParameters(DisplayConstants.CELL_WIDTH / 2,
//                               DisplayConstants.CELL_HEIGHT / 2,
//                               DisplayConstants.CELL_DEPTH);
    	
    	
//        if (this.isAlive()){
//            g.drawString("A"+ID,space.agentColors.getColor((isCooperative ? 3:0)));
//            g.drawFastRoundRect(space.agentColors.getColor((isCooperative ? 3:0)));
//            g.drawFastRoundRect(space.agentColors.getColor((isCooperative ? 3:0)));
//            if (taskCompleted==1)
//                g.drawFastRoundRect(COLOR[0]);
//            else if (taskAssigned==1 && taskCompleted==0)
//                g.drawFastRoundRect(COLOR[1]);
//            else if (contributedToTask==1)
//                g.drawFastRoundRect(COLOR[2]);
//            else
//                g.drawFastRoundRect(COLOR[3]);
//        }
        
        g.setDrawingParameters(DisplayConstants.CELL_WIDTH * (this.landParcelSize),
                DisplayConstants.CELL_HEIGHT * (this.landParcelSize), DisplayConstants.CELL_DEPTH);

//        try{
/*
 * use yieldBalancePerception as color
 */            
//        if (yieldBalancePerception.fuzzyMatch("high"))
//            g.drawStringInRect(colMapGYR.getColor(0), Color.black, "" + ID);
//        else if (yieldBalancePerception.fuzzyMatch("low")) {
//            g.drawStringInRect(colMapGYR.getColor(2), Color.black, "" + ID);
//            
//        }
//        else {
//            g.drawStringInRect(colMapGYR.getColor(1), Color.black, "" + ID);
//        }
//        } catch (FuzzyException e) {
//            e.printStackTrace();
//        }
        Color bc;
        if (yieldBalance>=lp.getPlantingCost())
            bc = Color.BLACK;
        else
            bc = Color.RED;
            
        g.drawStringInRect(colMapLRS.getColor((int)(this.lp.getTotalLRSCond()*100)), bc, "" + ID + "-" + df.format(lp.getLRSCond()));
//        g.drawStringInRect(colMapLRS.getColor((int)(this.lp.LRSCond*100)), bc, "" + ID);
        g.drawHollowFastRect(COLOR[this.LRSRule]);

        g.setDrawingParameters(DisplayConstants.CELL_WIDTH,
                DisplayConstants.CELL_HEIGHT, DisplayConstants.CELL_DEPTH);
//        g.drawFastRoundRect(Color.black);
    }

    /**
     * @return Returns the numFriendsPolled.
     */
    public int getNumFriendsPolled() {
        return numFriendsPolled;
    }

    /**
     * @param numFriendsPolled The numFriendsPolled to set.
     */
    public void setNumFriendsPolled(int numNeighborsPolled) {
        this.numFriendsPolled = numNeighborsPolled;
    }

    /**
     * @return Returns the strategySwitched.
     */
    public int getStrategySwitched() {
        return strategySwitched;
    }

    /**
     * @param strategySwitched The strategySwitched to set.
     */
    public void setStrategySwitched(int strategySwitched) {
        this.strategySwitched = strategySwitched;
    }

    /**
     * @return Returns the collaborationNode.
     */
    public SandboxNode getCollaborationNode() {
        return collaborationNode;
    }

    /**
     * @param collaborationNode The collaborationNode to set.
     */
    public void setCollaborationNode(SandboxNode strategyNode) {
        this.collaborationNode = strategyNode;
    }

    /**
     * @return Returns the numSupportersPolled.
     */
    public int getNumSupportersPolled() {
        return numSupportersPolled;
    }

    /**
     * @param numSupportersPolled The numSupportersPolled to set.
     */
    public void setNumSupportersPolled(int numSupportesPolled) {
        this.numSupportersPolled = numSupportesPolled;
    }

    /**
     * @return Returns the numCollEdgesAdded.
     */
    public int getCollEdgesAdded() {
        return collEdgesAdded;
    }


    /**
     * @param collEdgesAdded The numCollEdgesAdded to set.
     */
    public void setCollEdgesAdded(int numEdgesAdded) {
        this.collEdgesAdded = numEdgesAdded;
    }



    public int getNoFriends(){
        return this.acquaintancesNode.getOutDegree();
    }

    public int getNoCollaborators(){
        return this.collaborationNode.getOutDegree();
    }


    public int getAcqEdgesAdded() {
        return acqEdgesAdded;
    }
    /**
     * @param numAcqEdgesAdded The numAcqEdgesAdded to set.
     */
    public void setAcqEdgesAdded(int numAcqEdgesAdded) {
        this.acqEdgesAdded = numAcqEdgesAdded;
    }

    /**
     * @return Returns the model.
     */
    public Model getModel() {
        return model;
    }

    /**
     * @param model The model to set.
     */
    public void setModel(Model model) {
        this.model = model;
    }

    /**
     * @return Returns the numCollEdgesAddedCum.
     */
    public int getNumCollEdgesAdded() {
        return numCollEdgesAdded;
    }

    /**
     * @param numCollEdgesAddedCum The numCollEdgesAddedCum to set.
     */
    public void setNumCollEdgesAdded(int numCollEdgesAddedCum) {
        this.numCollEdgesAdded = numCollEdgesAddedCum;
    }

	public int getLandParcelSize() {
		return landParcelSize;
	}

    public void setLandParcelSize(int landParcelSize) {
		this.landParcelSize = landParcelSize;
	}


    public boolean isAlive(){
//        return resourceLevel>0.01;
        return true;
    }

    
    //for jess
    private PropertyChangeSupport pcs = new PropertyChangeSupport(this);

    public void addPropertyChangeListener(PropertyChangeListener p){
        pcs.addPropertyChangeListener(p);
    }
    
    public void removePropertyChangeListener(PropertyChangeListener p){
        pcs.removePropertyChangeListener(p);
    }

    private int maxYearsMemorised = 10;
    private double[] yieldBalanceMemory = new double[maxYearsMemorised];
    private double yieldBalance;
    private double lastYield = 0;

    
    private LandParcel lp;
    private boolean maintainLRS = false;
    private double lastMaintenanceTick = 0;
    
    //qualitative perceptions
    private String currentWeather;    
    private String currentLRSCond;
    public FuzzyVariable yieldBalanceFuzzy;
    private FuzzyValue yieldBalancePerception;
    
    public void initPerceptions(){
        yieldBalance = lp.getTotalArea()*6.0;
        //yieldBalance
        double xHigh[] = {yieldBalance, 1000};
        double yHigh[] = {0, 1};
        double xLow[] = {0, lp.getTotalArea()*4.0};
        double yLow[] = {1, 0};
        try {
            yieldBalanceFuzzy = new FuzzyVariable("yieldBalance", -100.0, 1000.0, "");
            yieldBalanceFuzzy.addTerm("high", xHigh, yHigh, 2);
            yieldBalanceFuzzy.addTerm("low", xLow, yLow, 2);
//            yieldBalanceFuzzy.addTerm("medium", "not high and not low");
            
            yieldBalancePerception = new FuzzyValue(yieldBalanceFuzzy, new SingletonFuzzySet(yieldBalance));
            FuzzyValue.setMatchThreshold(0.1);
        } catch (FuzzyException e) {
            e.printStackTrace();
        }
        
        for (int i=0;i<maxYearsMemorised;i++){
            yieldBalanceMemory[i] = yieldBalance;
        }
        
    }
    
    public void perceiveYieldBalance(){
        try {
            setYieldBalancePerception(new FuzzyValue(yieldBalanceFuzzy, new SingletonFuzzySet(yieldBalance)));
        } catch (FuzzyException e) {
            e.printStackTrace();
        }
    }
    
    public void memoriseYieldBalance(){
        for (int i=1;i<maxYearsMemorised;i++){
            yieldBalanceMemory[i] = yieldBalanceMemory[i-1];
        }
        yieldBalanceMemory[0] = yieldBalance;
    }

    private double lrsCostSubsidy = 0.0;
    public void payLrsCostSubsidy(){
        lrsCostSubsidy = model.getLrsCostSubsidySeasons()*lp.getLRSMaintenanceTime()*lp.getLRSMaintenanceCost();
    }
    private double lrsCostSubsidyReceived = 0.0;
    public void doLRSMaintenance (){
        if (this.lp.getLRSCond()<1.0){
            double mCost = lp.getLRSMaintenanceCost();
            double subsMCost;
            if (lrsCostSubsidy>=mCost){
                lrsCostSubsidy-=mCost;
                subsMCost=0.0;
            } else{
                subsMCost = mCost;
            }
            if ((lp.getCropLevel()==0 && lp.getPlantingCost()+subsMCost<yieldBalance)
                    ||(lp.getCropLevel()>0 && subsMCost<yieldBalance)){
                this.lp.maintainLRS(model.getTickCount());
                incYieldBalance(-subsMCost);
                setLastMaintenanceTick(model.getTickCount());
                if (subsMCost==0)
                    lrsCostSubsidyReceived+=mCost;
            }
        }
    }
    
    
    public void plantCrop(){
        if (yieldBalance>lp.getPlantingCost()){
            lp.plantCrop();
            incYieldBalance(-lp.getPlantingCost());
        } else {
            lp.setCropLevel(0.0);
        }
    }
    
    private double floodingDroughtCompensationReceived = 0.0;
    public void harvestCrop(){
        if (lp.getCropLevel()>0){
            /*
             * amount of subsidy is a percentage of the maximum possible yield (=lp.calcYield(0))
             */
            double subsYield = lp.calcYield(0)*model.getFloodingDroughtCompensationProp();
            double yield = lp.calcYield(space.getPropMaxLoss());
            /*
             * pay subsidy if:
             *  -weather is not normal (ie dry or wet)
             *  -agent is currently not maintaining
             *  -subsidised yield is more than actual yield
             */
            if (space.getClimate().currentWeather.getRaintype()!=Weather.NORMAL
//                    && !this.maintainLRS
                    && subsYield>yield){
                floodingDroughtCompensationReceived+=subsYield-yield;
                yield=subsYield;
            }
            lp.harvestCrop();
            lastYield = yield;
            incYieldBalance(lastYield);
        }
        else{
            lastYield = 0;
        }
    }
    
    /**
     * @return Returns the yieldBalance.
     */
    public double getYieldBalance() {
        return yieldBalance;
    }

    /**
     * @param yieldBalance The yieldBalance to set.
     */
    public void setYieldBalance(double yieldBalance) {
        double old = this.yieldBalance;
        this.yieldBalance = yieldBalance;
        pcs.firePropertyChange("yieldBalance", old, this.yieldBalance);
    }
    
    public void incYieldBalance(double inc){
        setYieldBalance(this.yieldBalance+inc);
    }

    /**
     * @return Returns the maintainLRS.
     */
    public boolean isMaintainLRS() {
        return maintainLRS;
    }

    /**
     * @param maintainLRS The maintainLRS to set.
     */
    public void setMaintainLRS(boolean maintainLRS) {
        boolean old = this.maintainLRS;
        this.maintainLRS = maintainLRS;
        pcs.firePropertyChange("maintainLRS", old, this.maintainLRS);
    }
    
    public void perceiveCurrentWeather(){
        int weather = this.space.getClimate().currentWeather.getRaintype();
        if (weather == Weather.DRY)
            setCurrentWeather("dry");
        else if (weather == Weather.WET)
            setCurrentWeather("wet");
        else
            setCurrentWeather("normal");
    }
    
    public void perceiveCurrentLRSCond(){
        double LRSCond = this.lp.getLRSCond();
        if (LRSCond >= 0.95)
            setCurrentLRSCond("good");
        else if (LRSCond >= 0.90)
            setCurrentLRSCond("medium");
        else
            setCurrentLRSCond("bad");
    }
    
    public String getCurrentWeather(){
        return currentWeather;
    }

    public void setCurrentWeather(String currentWeather) {
        String old = this.currentWeather;
        this.currentWeather = currentWeather;
        pcs.firePropertyChange("currentWeather", old, this.currentWeather);
    }

    public String getCurrentLRSCond(){
        return currentLRSCond;
    }

    public void setCurrentLRSCond(String currentLRSCond) {
        String old = this.currentLRSCond;
        this.currentLRSCond = currentLRSCond;
        pcs.firePropertyChange("currentLRSCond", old, this.currentLRSCond);
    }

    /**
     * @return Returns the lastMaintenanceTick.
     */
    public double getLastMaintenanceTick() {
        return lastMaintenanceTick;
    }

    /**
     * @param lastMaintenanceTick The lastMaintenanceTick to set.
     */
    public void setLastMaintenanceTick(double lastMaintenanceTick) {
        double old = this.lastMaintenanceTick;
        this.lastMaintenanceTick = lastMaintenanceTick;
        pcs.firePropertyChange("lastMaintenanceTick", old, this.lastMaintenanceTick);
    }

    /**
     * @return Returns the lp.
     */
    public LandParcel getLp() {
        return lp;
    }

    /**
     * @param lp The lp to set.
     */
    public void setLp(LandParcel lp) {
        this.lp = lp;
    }

    /**
     * @return the yieldBalancePerception
     */
    public FuzzyValue getYieldBalancePerception() {
        return yieldBalancePerception;
    }

    /**
     * @param yieldBalancePerception the yieldBalancePerception to set
     */
    public void setYieldBalancePerception(FuzzyValue yieldBalancePerception) {
        FuzzyValue old = this.yieldBalancePerception;
//        if (!old.equalsStar(yieldBalancePerception)){
//            logger.printIt("fire");
        this.yieldBalancePerception = yieldBalancePerception;
            pcs.firePropertyChange("yieldBalancePerception", old, yieldBalancePerception);
//        }
    }

/*
 * decision algorithm for LRSRules
 */
    //number of LRSRule, used as tag, see agentDecisionRules.clp
    private int LRSRule = 1;
    //rating of those agents that use the same LRSRule
    double ratingMe;
    //rating of those agents that use a different LRSRule (presently only other rule is supported)
    double ratingOthers;
    //rating criterium
    public double calcRatingScore(){
        return(yieldBalanceMemory[1]);
    }
    public void estimateRating(){
        estimateRating(acquaintancesNode.getToNodes());
    }
    private boolean estimationDue = false;
    private double cmpCoefficient = 0.90;
    private void estimateRating(ArrayList<SandboxNode> nodeList){
        double totalRatingWe = this.ratingMe;
        double totalResourceLevelWe = this.calcRatingScore();
        double totalRatingOthers = this.ratingOthers;
        double totalResourceLevelOthers = 0;
        double cntWe = 1,cntOthers = 0;
        int aDifferentLRSRule = 0;
        if (!nodeList.isEmpty()) {
            for (SandboxNode n : nodeList) {
                if (this.getLRSRule()==n.getAgent().getLRSRule()){
                    totalRatingWe+=n.getAgent().getRatingMe();
                    totalRatingOthers+=n.getAgent().getRatingOthers();
                    totalResourceLevelWe+=n.getAgent().calcRatingScore();
                    cntWe++;
                    this.collaborationNode.makeEdgeTo(n.getAgent().getCollaborationNode());
                }
                else{
                    aDifferentLRSRule = n.getAgent().getLRSRule();
                    totalResourceLevelOthers+=n.getAgent().calcRatingScore();
                    cntOthers++;
                }
            }
            double avgRatingWe = totalRatingWe/cntWe;
            double avgRatingOthers  = totalRatingOthers/cntWe;
            double avgResourceLevelWe = totalResourceLevelWe/cntWe;
            double avgResourceLevelOthers = 0;
            if (cntOthers>0){
                avgResourceLevelOthers = totalResourceLevelOthers/cntOthers;
            }
            if (avgResourceLevelWe<cmpCoefficient*avgResourceLevelOthers){
                this.ratingMe*=cmpCoefficient;
                this.ratingOthers*=2.0-cmpCoefficient;
            }
            else{
                this.ratingMe=avgRatingWe;
                this.ratingOthers=avgRatingOthers;
            }
            if (ratingMe<cmpCoefficient*ratingOthers){
                switchRule(aDifferentLRSRule);
                leaveCollaborationNetwork();
            }

        }
    }
    
    public void switchRule(int newRule){
        this.setLRSRule(newRule);
        double tmp = this.ratingMe;
        this.ratingMe = this.ratingOthers;
        this.ratingOthers = tmp;
    }
        
    
    /**
     * @return the componentID
     */
    public int getComponentID() {
        return componentID;
    }

    /**
     * @param componentID the componentID to set
     */
    public void setComponentID(int componentID) {
        this.componentID = componentID;
    }

    /**
     * @return the dependencyNode
     */
    public SandboxNode getDependencyNode() {
        return dependencyNode;
    }

    /**
     * @param dependencyNode the dependencyNode to set
     */
    public void setDependencyNode(SandboxNode dependencyNode) {
        this.dependencyNode = dependencyNode;
    }

    /**
     * @return the lRSRule
     */
    public int getLRSRule() {
        return LRSRule;
    }

    /**
     * @param rule the lRSRule to set
     */
    public void setLRSRule(int rule) {
        LRSRule = rule;
    }

    /**
     * @return the lastYield
     */
    public double getLastYield() {
        return lastYield;
    }

    /**
     * @param lastYield the lastYield to set
     */
    public void setLastYield(double lastYield) {
        this.lastYield = lastYield;
    }

    /**
     * @return the ratingMe
     */
    public double getRatingMe() {
        return ratingMe;
    }

    /**
     * @param ratingMe the ratingMe to set
     */
    public void setRatingMe(double ratingMe) {
        this.ratingMe = ratingMe;
    }

    /**
     * @return the ratingOthers
     */
    public double getRatingOthers() {
        return ratingOthers;
    }

    /**
     * @param ratingOthers the ratingOthers to set
     */
    public void setRatingOthers(double ratingOthers) {
        this.ratingOthers = ratingOthers;
    }

    /**
     * @return the space
     */
    public Space getSpace() {
        return space;
    }

    /**
     * @return the estimationDue
     */
    public boolean isEstimationDue() {
        return estimationDue;
    }

    /**
     * @param estimationDue the estimationDue to set
     */
    public void setEstimationDue(boolean estimationDue) {
        boolean old = this.estimationDue;
        this.estimationDue = estimationDue;
        pcs.firePropertyChange("estimationDue", old, estimationDue);
    }

    /**
     * @param lrsCostSubsidy the lrsCostSubsidy to set
     */
    public void setLrsCostSubsidy(double lrsCostSubsidy) {
        this.lrsCostSubsidy = lrsCostSubsidy;
    }

    /**
     * @return the floodingDroughtCompensationReceived
     */
    public double getFloodingDroughtCompensationReceived() {
        return floodingDroughtCompensationReceived;
    }

    /**
     * @return the lrsCostSubsidyReceived
     */
    public double getLrsCostSubsidyReceived() {
        return lrsCostSubsidyReceived;
    }

}
