/*$$
 * Copyright (c) 1999, Trustees of the University of Chicago
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with 
 * or without modification, are permitted provided that the following 
 * conditions are met:
 *
 *	 Redistributions of source code must retain the above copyright notice,
 *	 this list of conditions and the following disclaimer.
 *
 *	 Redistributions in binary form must reproduce the above copyright notice,
 *	 this list of conditions and the following disclaimer in the documentation
 *	 and/or other materials provided with the distribution.
 *
 * Neither the name of the University of Chicago nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE TRUSTEES OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *$$*/
package de.uni_kassel.usf.LandReclamationJess;

import java.awt.Color;

import uchicago.src.sim.gui.DrawableEdge;
import uchicago.src.sim.gui.SimGraphics;
import uchicago.src.sim.network.DefaultEdge;
import uchicago.src.sim.network.Node;

/**
 * The edge between JinGirNewNodes.
 * 
 * @version $Revision: 1.3 $ $Date: 2004/11/03 19:51:02 $
 */
public class SandboxEdge extends DefaultEdge implements DrawableEdge {

    private double initStrength = 1;

    private int age = 0;

    private int timesUsed = 0;

    private double reliability;

    public SandboxEdge() {
    }

    public void resetStrength() {
        setStrength(initStrength);
    }

    public SandboxEdge(Node from, Node to, double strength) {
        super(from, to, "");
        initStrength = strength;
        resetStrength();
        reliability = 1.0;
    }

    public void decay() {
        strength -= 1;
    }

    public void age() {
        age++;
    }

    public void incTimesUsed() {
        timesUsed++;
    }

    private static final Color[] COLOR = { Color.green, Color.yellow, Color.red };

    public void draw(SimGraphics g, int fromX, int toX, int fromY, int toY) {
        if (strength >= initStrength / 3 * 2)
            g.drawDirectedLink(COLOR[0], fromX, toX, fromY, toY);
        else if (strength >= initStrength / 3)
            g.drawDirectedLink(COLOR[1], fromX, toX, fromY, toY);
        else
            g.drawDirectedLink(COLOR[2], fromX, toX, fromY, toY);
    }

    /**
     * @param initStrength
     *            The initStrength to set.
     */
    public void setInitStrength(double initStrength) {
        this.initStrength = initStrength;
    }

    /**
     * @return Returns the age.
     */
    public double getAge() {
        return age;
    }

    /**
     * @param age
     *            The age to set.
     */
    public void setAge(int age) {
        this.age = age;
    }

    /**
     * @return Returns the timesUsed.
     */
    public double getTimesUsed() {
        return timesUsed;
    }

    /**
     * @param timesUsed
     *            The timesUsed to set.
     */
    public void setTimesUsed(int timesUsed) {
        this.timesUsed = timesUsed;
    }

    /**
     * @return Returns the reliability.
     */
    public double getReliability() {
        return reliability;
    }

    /**
     * @param reliability The reliability to set.
     */
    public void setReliability(double reliability) {
        this.reliability = reliability;
    }
}
