/*$$
 * Copyright (c) 1999, Trustees of the University of Chicago
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with 
 * or without modification, are permitted provided that the following 
 * conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 * Neither the name of the University of Chicago nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE TRUSTEES OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *$$*/
package de.uni_kassel.usf.LandReclamationJess;

import java.awt.Color;
import java.util.ArrayList;

import uchicago.src.collection.RangeMap;
import uchicago.src.sim.gui.Drawable;
import uchicago.src.sim.gui.RectNetworkItem;
import uchicago.src.sim.gui.SimGraphics;
import uchicago.src.sim.network.DefaultDrawableNode;
import uchicago.src.sim.network.DefaultNode;
import uchicago.src.sim.network.Edge;
import uchicago.src.sim.util.Random;

/**
 * Node class. SandboxNode extends
 * DefaultDrawableNode and so it is both a DefaultNode and a
 * DrawableNonGridNode. All of the important behavoir here as to do with
 * creating new links.
 * @version
 */

public class SandboxNode extends DefaultDrawableNode {

    private int maxDegree = 100;
    
    private double initEdgeStrength = 100;
    private static double d = 8;
    private double beta = initEdgeStrength;
    private double alpha = -Math.log(0.5)/d;

    private int numEdgesRemoved = 0;
    
    private SandboxAgent agent;
    
    static final int acquaintaincesSemantics = 1;
    static final int coworkerSemantics = 2;
    static final int dependencySemantics = 3;
    private int semantics;
    
    
    public int getInOutDegree(){
        return this.getInDegree()+this.getOutDegree();
    }
    /*
     * No arg constructor so that a SandboxNode can be created from a file.
     */

    public SandboxNode() {
    }

    public SandboxNode(SandboxAgent a) {
        setAgent(a);
    }

    public SandboxNode(SandboxAgent a,int maxDegree, double initEdgeStrength) {
        setAgent(a);
        init(0, 0);
        this.maxDegree = maxDegree;
        this.initEdgeStrength = initEdgeStrength;
//        beta = initEdgeStrength;
//        alpha = -Math.log(0.5)/d;
    }

    /**
     * Initialize this SandboxNode, this takes the place of the constructor when
     * a SandboxNode is created from a file using its no-arg constructor.
     */
    public void init(int x, int y) {
        RectNetworkItem rect = new RectNetworkItem(x, y);
        setDrawable(rect);
    }

    /**
     * @return Returns the agent.
     */
    public SandboxAgent getAgent() {
        return agent;
    }

    /**
     * @param agent The agent to set.
     */
    public void setAgent(SandboxAgent agent) {
        this.agent = agent;
    }
    
    public double getDistanceStrength(SandboxAgent a,SandboxAgent b){
        return beta*Math.exp(-alpha*Space.getDistanceBetween(a,b));
    }
    /**
     * Makes an edge to the specified node and from the specifed node to
     * this SandboxNode if both nodes do not already have edges to each other and
     * if adding the edge keeps their degrees less than maxDegree. If an edge allready
     * exitsts, its strength is refreshed.
     * 
     * @param node
     * @param maxDegree
     */
    public boolean makeEdgeToFrom(DefaultNode node) {
        boolean edgeConstructed = false;
        if (!node.equals(this)) {
            if (hasEdgeTo(node)) {
                ArrayList<SandboxEdge> outE = new ArrayList(getOutEdges());
                for (SandboxEdge e : outE) {
                    if (node.equals(e.getTo())) {
                        e.resetStrength();
                    }
                }
                outE = new ArrayList(node.getOutEdges());
                for (SandboxEdge e : outE) {
                    if (this.equals(e.getTo())) {
                        e.resetStrength();
                    }
                }
            } else if (getOutDegree() < maxDegree
                    && node.getOutDegree() < maxDegree) {
//                double newEdgeStrength = getEdgeIntensity(this.agent,((SandboxNode)node).getAgent());
                double newEdgeStrength = initEdgeStrength;
                Edge edge = new SandboxEdge(this, node, newEdgeStrength);
                addOutEdge(edge);
                node.addInEdge(edge);
                Edge otherEdge = new SandboxEdge(node, this, newEdgeStrength);
                node.addOutEdge(otherEdge);
                addInEdge(otherEdge);
                edgeConstructed = true;
            }
        }
        return edgeConstructed;
    }

    /**
     * Makes an edge to the specified node
     * If the edge allready exitsts, its strength is refreshed.
     * if the edge does not exist the a new out edge is constrcted
     * if adding the edge keeps the (out) degree less than maxDegree. 
     * 
     * @param node
     * @param maxDegree
     */
    public boolean makeEdgeTo(DefaultNode node) {
        boolean edgeConstructed = false;
        if (!node.equals(this)) {
            if (hasEdgeTo(node)) {
                ArrayList<SandboxEdge> outE = new ArrayList(getOutEdges());
                for (SandboxEdge e : outE) {
                    if (node.equals(e.getTo())) {
                        e.resetStrength();
                        e.incTimesUsed();
                    }
                }
            } else
            if (getOutDegree() < maxDegree
                    && node.getInDegree() < maxDegree) {
//                double newEdgeStrength = getDistanceStrength(this.agent,((SandboxNode)node).getAgent());
                double newEdgeStrength = initEdgeStrength;
                SandboxEdge edge = new SandboxEdge(this, node, newEdgeStrength);
                edge.setTimesUsed(1);
                edge.setAge(0);
                addOutEdge(edge);
                node.addInEdge(edge);
                this.agent.getModel().getEdgeList().add(edge);
                edgeConstructed = true;
            }
        }
        return edgeConstructed;
    }

    /**
     * Decays the strengh of all edges out of this node by subtracting 1 from the strength of the edges.
     * Edges with strength of 0 are removed.
     * 
     */
    public void decayEdges() {
        numEdgesRemoved = 0;
        ArrayList<SandboxEdge> outE = new ArrayList(this.getOutEdges());
        for (SandboxEdge e : outE) {
            e.decay();
            if (e.getStrength() <= 0) {
                this.removeOutEdge(e);
                e.getTo().removeInEdge(e);
                this.agent.getModel().getEdgeList().remove(e);
                numEdgesRemoved++;
            }
        }
    }

    public void ageEdges() {
        ArrayList<SandboxEdge> outE = new ArrayList(this.getOutEdges());
        for (SandboxEdge e : outE) {
            e.age();
        }
    }

    /**
     * Resets the strengh of all edges out of this node.
     * 
     */
    public void resetEdgeStrenghts() {
        ArrayList<SandboxEdge> outE = new ArrayList(this.getOutEdges());
        for (SandboxEdge e : outE) {
            e.resetStrength();
        }
    }

    public boolean meetRandom(ArrayList<SandboxNode> nodeList) {
        boolean metNew = false;
        if (!nodeList.isEmpty()) {

            RangeMap map = new RangeMap();
            int degree;
            int sumDegree = 0;
            for (SandboxNode n:nodeList){
                degree = n.getInDegree();
                if (degree>0){
                    map.put(sumDegree,n);
                    sumDegree+=degree;
                }
            }
            SandboxNode rndNode;
            if (sumDegree>0){
                rndNode = (SandboxNode) map.get(Random.uniform.nextIntFromTo(0,sumDegree));                
            } else{
                rndNode = nodeList.get(Random.uniform.nextIntFromTo(0,nodeList.size()-1));
            }
                
            if (this.makeEdgeTo(rndNode)) {
                metNew = true;
            }
            rndNode.makeEdgeTo(this);
        }
        return metNew;
    }
    
    public boolean introduceNeighbors() {
        boolean edgeAdded = false;
        if (getOutDegree() > 1) {
            SandboxNode jNode = (SandboxNode) getRandomNodeOut();
            SandboxNode kNode = (SandboxNode) getRandomNodeOut();
            while (jNode.equals(kNode)) {
                kNode = (SandboxNode) getRandomNodeOut();
            }
            edgeAdded = jNode.makeEdgeTo(kNode)||kNode.makeEdgeTo(jNode);
        }

        return edgeAdded;
    }
    

// /**
//     * Creates a new edge between this SandboxNode and a node chosen at random
//     * from the specified list. This edge is created via <tt>makeEdgeFromTo</tt>
//     * and so those conditions must be met as well.
//     * 
//     * @param list
//     * @param maxDegree
//     */
//    public void meetRandom(ArrayList list, int maxDegree) {
//        int index = Random.uniform.nextIntFromTo(0, list.size() - 1);
//        SandboxNode node = (SandboxNode) list.get(index);
//        while (this.equals(node)) {
//            index = Random.uniform.nextIntFromTo(0, list.size() - 1);
//            node = (SandboxNode) list.get(index);
//        }
//
//        makeEdgeToFrom(node, maxDegree);
//    }
//
//    /**
//     * Creates an edge between two nodes that both link to this SandboxNode.
//     * This edge is created via <tt>makeEdgeFromTo</tt> and so those
//     * conditions must be met as well.
//     * 
//     * @param maxDegree
//     */
//    public void meetNeighbor(int maxDegree) {
//
//        if (getOutDegree() > 1) {
//            SandboxNode jNode = (SandboxNode) getRandomNodeOut();
//            SandboxNode kNode = (SandboxNode) getRandomNodeOut();
//
//            while (jNode.equals(kNode)) {
//                kNode = (SandboxNode) getRandomNodeOut();
//            }
//
//            jNode.makeEdgeToFrom(kNode, maxDegree);
//        }
//    }
//
//    public SandboxNode meetNeighborsNeighbor(int maxDegree) {
//        SandboxNode nNeighbor = this;
//        if (getOutDegree() > 1) {
//            nNeighbor = (SandboxNode) getRandomNodeOut();
//
//            nNeighbor.makeEdgeToFrom(this, maxDegree);
//        }
//        return nNeighbor;
//    }
//
//    /**
//     * Removes a link between this SandboxNode and one chosen at random from
//     * those linked to this SandboxNode.
//     */
//    public void removeFriend() {
//        SandboxNode jNode = (SandboxNode) getRandomNodeOut();
//
//        // will be null if no outEdges.
//        if (jNode != null) {
//            removeEdgesTo(jNode);
//            jNode.removeEdgesFrom(this);
//            removeEdgesFrom(jNode);
//            jNode.removeEdgesTo(this);
//        }
//    }

    /**
     * @return Returns the maxDegree.
     */
    public int getMaxDegree() {
        return maxDegree;
    }

    /**
     * @param maxDegree The maxDegree to set.
     */
    public void setMaxDegree(int maxDegree) {
        this.maxDegree = maxDegree;
    }

    /**
     * @return Returns the numEdgesRemoved.
     */
    public int getNumEdgesRemoved() {
        return numEdgesRemoved;
    }

    /**
     * @param numEdgesRemoved The numEdgesRemoved to set.
     */
    public void setNumEdgesRemoved(int numEdgesRemoved) {
        this.numEdgesRemoved = numEdgesRemoved;
    }
    /**
     * @return Returns the semantics.
     */
    public int getSemantics() {
        return semantics;
    }
    /**
     * @param semantics The semantics to set.
     */
    public void setSemantics(int semantics) {
        this.semantics = semantics;
    }



}
