package de.uni_kassel.usf.LandReclamationJess;

import java.util.*;

public class Task {
    
    private ArrayList<ExpertiseAtom> requiredExpertise;
    private double associatedGain;
    boolean pending;
    
    public Task(){
        requiredExpertise = new ArrayList<ExpertiseAtom>();
        pending = false;
    }
    
    public void setPending(){
        boolean atomUnavailable = false;
        if (!requiredExpertise.isEmpty()){
            Iterator it = requiredExpertise.iterator();
            do{
                atomUnavailable = !(((ExpertiseAtom) it.next()).isAvailable()); 
            }
            while (!atomUnavailable && it.hasNext());
        }
        pending = atomUnavailable;
    }

    /**
     * @return Returns true if at least one ExpertiseAtom of this Task is missing
     */
    public boolean isPending(){
        return pending;
    }
    /**
     * marks this task completed ignoring whether the ExpertiseAtoms are available
     *
     */
    public void markCompleted(){
        pending = false;
    }
 
    /**
     * @return Returns true if the ExpertiseAtom with ID aID was perviously unavailable and is now made available.
     */
    
    public boolean markAtomAvailable(int aID){
        boolean success = false;
        if (!requiredExpertise.isEmpty()){
            Iterator<ExpertiseAtom> it = requiredExpertise.iterator();
            ExpertiseAtom a;
            do {
                a = it.next();
                if (a.getID() == aID){
                    success = a.markAvailable(); 
                }
            }while (!success && it.hasNext());

            setPending();
        }
        return success;
    }
    
    public boolean markAtomUnavailable(int aID){
        boolean success = false;
        if (!requiredExpertise.isEmpty()){
            Iterator it = requiredExpertise.iterator();
            ExpertiseAtom a;
            do {
                a = (ExpertiseAtom) it.next();
                if (a.getID() == aID){
                    success = a.markUnavailable();
                }
            }while (!success && it.hasNext());

            setPending();
        }
        return success;
    }

    public void markAllAtomsUnavailable(){
        if (!requiredExpertise.isEmpty()){
            Iterator it = requiredExpertise.iterator();
            ExpertiseAtom a;
            do {
                a = (ExpertiseAtom) it.next();                  
                a.markUnavailable();
            }while (it.hasNext());

            setPending();
        }
    }
    
    public boolean isAtomRequired(int aID){
        boolean success = false;
        if (!requiredExpertise.isEmpty()){
            Iterator it = requiredExpertise.iterator();
            ExpertiseAtom a;
            do {
                a = (ExpertiseAtom) it.next();
                if (a.getID() == aID && !a.isAvailable()){
                    success = true; 
                }
            }while (!success && it.hasNext());

        }
        return success;
    }
    
    public void markUncompleted(){
        pending = true;
    }
    
    public void addExpertiseAtom(int aID){
        requiredExpertise.add(new ExpertiseAtom(aID));
    }
    
    public ExpertiseAtom getExpertiseAtomAt(int pos){
        return requiredExpertise.get(pos);
    }

    /**
     * @return Returns the associatedGain.
     */
    public double getAssociatedGain() {
        return associatedGain;
    }

    /**
     * @param associatedGain The associatedGain to set.
     */
    public void setAssociatedGain(double associatedGain) {
        this.associatedGain = associatedGain;
    }
    
}
